#
# Cognica
#
# Copyright (c) 2023 Cognica, Inc.
#

# pylint: disable=no-member,broad-exception-caught
# pylint: disable=missing-class-docstring,missing-function-docstring

import time
import typing as t

import grpc

from cognica.channel import Channel
from cognica.protobuf import key_value_db_pb2, key_value_db_pb2_grpc


messages: t.TypeAlias = key_value_db_pb2  # type: ignore

KeyValueDBServiceStub = key_value_db_pb2_grpc.KeyValueDBServiceStub
KeyspaceManagerServiceStub = key_value_db_pb2_grpc.KeyspaceManagerServiceStub

StatusType: t.TypeAlias = messages.StatusType  # type: ignore
Response: t.TypeAlias = messages.Response  # type: ignore
PutRequest: t.TypeAlias = messages.PutRequest  # type: ignore
PutResponse: t.TypeAlias = messages.PutResponse  # type: ignore
RemoveRequest: t.TypeAlias = messages.RemoveRequest  # type: ignore
RemoveResponse: t.TypeAlias = messages.RemoveResponse  # type: ignore
GetRequest: t.TypeAlias = messages.GetRequest  # type: ignore
GetResponse: t.TypeAlias = messages.GetResponse  # type: ignore
MultiGetRequest: t.TypeAlias = messages.MultiGetRequest  # type: ignore
MultiGetResponse: t.TypeAlias = messages.MultiGetResponse  # type: ignore
BatchedPutRequest: t.TypeAlias = messages.BatchedPutRequest  # type: ignore
BatchedPutResponse: t.TypeAlias = messages.BatchedPutResponse  # type: ignore
BatchedRemoveRequest: t.TypeAlias = messages.BatchedRemoveRequest  # type: ignore
BatchedRemoveResponse: t.TypeAlias = (
    messages.BatchedRemoveResponse  # type: ignore
)
BatchedGetRequest: t.TypeAlias = messages.BatchedGetRequest  # type: ignore
BatchedGetResponse: t.TypeAlias = messages.BatchedGetResponse  # type: ignore

CreateKeyspaceRequest: t.TypeAlias = (
    messages.CreateKeyspaceRequest  # type: ignore
)
CreateKeyspaceResponse: t.TypeAlias = (
    messages.CreateKeyspaceResponse  # type: ignore
)
DropKeyspaceRequest: t.TypeAlias = messages.DropKeyspaceRequest  # type: ignore
DropKeyspaceResponse: t.TypeAlias = (
    messages.DropKeyspaceResponse  # type: ignore
)
TruncateKeyspaceRequest: t.TypeAlias = (
    messages.TruncateKeyspaceRequest  # type: ignore
)
TruncateKeyspaceResponse: t.TypeAlias = (
    messages.TruncateKeyspaceResponse  # type: ignore
)
ListKeyspacesRequest: t.TypeAlias = (
    messages.ListKeyspacesRequest  # type: ignore
)
ListKeyspacesResponse: t.TypeAlias = (
    messages.ListKeyspacesResponse  # type: ignore
)


def _create_stub(stub: t.Callable, channel: Channel):
    stub = stub(channel.channel)

    return stub


class KeyValueDB:
    def __init__(self, channel: Channel, timeout: int | None = None):
        """
        parameter 'timeout' is in seconds
        """

        self._max_retry_count = 5
        self._stub = _create_stub(KeyValueDBServiceStub, channel)
        self._timeout = timeout

    def put(
        self,
        keyspace_name: str,
        key: t.Union[bytes, str],
        value: t.Union[bytes, str],
        ttl: int = 0,
        create_if_missing: bool = True,
    ) -> bool:
        if isinstance(key, str):
            key = key.encode("utf-8")
        if isinstance(value, str):
            value = value.encode("utf-8")

        req = PutRequest(
            keyspace_name=keyspace_name,
            key=key,
            value=value,
            ttl=ttl,
            create_if_missing=create_if_missing,
        )
        try:
            resp: PutResponse = self._invoke(
                self._stub.put, req, wait_for_ready=True
            )
        except Exception:
            return False

        return resp.response.status == StatusType.kOK

    def remove(self, keyspace_name: str, key: t.Union[bytes, str]) -> bool:
        if isinstance(key, str):
            key = key.encode("utf-8")

        req = RemoveRequest(keyspace_name=keyspace_name, key=key)
        try:
            resp: RemoveResponse = self._invoke(
                self._stub.remove, req, wait_for_ready=True
            )
        except Exception:
            return False

        return resp.response.status == StatusType.kOK

    def get(
        self, keyspace_name: str, key: t.Union[bytes, str]
    ) -> t.Optional[bytes]:
        if isinstance(key, str):
            key = key.encode("utf-8")

        req = GetRequest(keyspace_name=keyspace_name, key=key)
        try:
            resp: GetResponse = self._invoke(
                self._stub.get, req, wait_for_ready=True
            )
        except Exception:
            return None

        if resp.response.status != StatusType.kOK:
            return None

        return resp.value

    def mget(
        self, keyspace_name: str, keys: t.List[t.Union[bytes, str]]
    ) -> t.List[t.Optional[bytes]]:
        keys_encoded = []
        for key in keys:
            if isinstance(key, str):
                keys_encoded.append(key.encode("utf-8"))
            elif isinstance(key, bytes):
                keys_encoded.append(key)
            else:
                raise TypeError("key must be either string or bytes type.")

        req = MultiGetRequest(keyspace_name=keyspace_name, keys=keys_encoded)
        try:
            resp: MultiGetResponse = self._invoke(
                self._stub.mget, req, wait_for_ready=True
            )
        except Exception:
            return [None] * len(keys)

        values = []
        for response, value in zip(resp.responses, resp.values):
            if response.status == StatusType.kOK:
                values.append(value)
            else:
                values.append(None)

        return values

    def put_batch(
        self,
        keyspace_name: str,
        keys: t.List[t.Union[bytes, str]],
        values: t.List[t.Union[bytes, str]],
        ttls: t.List[int],
        create_if_missing: bool = True,
    ) -> t.List[bool]:
        keys_encoded = []
        values_encoded = []
        for key, value in zip(keys, values):
            if isinstance(key, str):
                key = key.encode("utf-8")
            if isinstance(key, str):
                value = value.encode("utf-8")  # type: ignore
            keys_encoded.append(key)
            values_encoded.append(value)

        req = BatchedPutRequest(
            keyspace_name=keyspace_name,
            keys=keys_encoded,
            values=values_encoded,
            ttls=ttls,
            create_if_missing=create_if_missing,
        )
        try:
            resp: BatchedPutResponse = self._invoke(
                self._stub.put_batch, req, wait_for_ready=True
            )
        except Exception:
            return [False] * len(keys)

        statuses = []
        for resp in resp.responses:
            if resp.status == StatusType.kOK:
                statuses.append(True)
            else:
                statuses.append(False)

        return statuses

    def remove_batch(
        self, keyspace_name: str, keys: t.List[t.Union[bytes, str]]
    ) -> t.List[bool]:
        keys_encoded = []
        for key in keys:
            if isinstance(key, str):
                key = key.encode("utf-8")
            keys_encoded.append(key)

        req = BatchedRemoveRequest(
            keyspace_name=keyspace_name, keys=keys_encoded
        )
        try:
            resp: BatchedRemoveResponse = self._invoke(
                self._stub.remove_batch, req, wait_for_ready=True
            )
        except Exception:
            return [False] * len(keys)

        statuses = []
        for resp in resp.responses:
            if resp.status == StatusType.kOK:
                statuses.append(True)
            else:
                statuses.append(False)

        return statuses

    def get_batch(
        self, keyspace_name: str, keys: t.List[t.Union[bytes, str]]
    ) -> t.List[t.Optional[bytes]]:
        keys_encoded = []
        for key in keys:
            if isinstance(key, str):
                key = key.encode("utf-8")
            keys_encoded.append(key)

        req = BatchedGetRequest(keyspace_name=keyspace_name, keys=keys_encoded)
        try:
            resp: BatchedGetResponse = self._invoke(
                self._stub.get_batch, req, wait_for_ready=True
            )
        except Exception:
            return [None] * len(keys)

        values = []
        for resp, value in zip(resp.responses, resp.values):
            if resp.response.status == StatusType.kOK:
                values.append(value)
            else:
                values.append(None)

        return values

    def _invoke(self, func, *args, **kwargs):
        retry = 0
        backoff = 0.1
        resp = None
        if self._timeout:
            kwargs["timeout"] = self._timeout
        while retry < self._max_retry_count:
            try:
                resp = func(*args, **kwargs)
            except grpc.RpcError as rpc_error:
                if rpc_error.code() == grpc.StatusCode.UNAVAILABLE:  # type: ignore
                    retry += 1
                    time.sleep(backoff)
                    backoff *= 2
                else:
                    raise
            else:
                break

        return resp


class KeyspaceManager:
    def __init__(self, channel: Channel, timeout: int | None = None):
        self._max_retry_count = 5
        self._stub = _create_stub(KeyspaceManagerServiceStub, channel)
        self._timeout = timeout

    def create_keyspace(self, keyspace_name: str) -> bool:
        req = CreateKeyspaceRequest(keyspace_name=keyspace_name)
        resp: CreateKeyspaceResponse = self._invoke(
            self._stub.create_keyspace, req, wait_for_ready=True
        )

        return resp.response.status == StatusType.kOK

    def drop_keyspace(self, keyspace_name: str) -> bool:
        req = DropKeyspaceRequest(keyspace_name=keyspace_name)
        resp: DropKeyspaceResponse = self._invoke(
            self._stub.drop_keyspace, req, wait_for_ready=True
        )

        return resp.response.status == StatusType.kOK

    def truncate_keyspace(self, keyspace_name: str) -> bool:
        req = TruncateKeyspaceRequest(keyspace_name=keyspace_name)
        resp: TruncateKeyspaceResponse = self._invoke(
            self._stub.truncate_keyspace, req, wait_for_ready=True
        )

        return resp.response.status == StatusType.kOK

    def list_keyspaces(self) -> t.List[str]:
        req = ListKeyspacesRequest()
        resp: ListKeyspacesResponse = self._invoke(
            self._stub.list_keyspaces, req, wait_for_ready=True
        )

        if resp.response.status != StatusType.kOK:
            return []

        return resp.keyspace_names

    def _invoke(self, func, *args, **kwargs):
        retry = 0
        backoff = 0.1
        resp = None
        if self._timeout:
            kwargs["timeout"] = self._timeout
        while retry < self._max_retry_count:
            try:
                resp = func(*args, **kwargs)
            except grpc.RpcError as rpc_error:
                if rpc_error.code() == grpc.StatusCode.UNAVAILABLE:  # type: ignore
                    retry += 1
                    time.sleep(backoff)
                    backoff *= 2
                else:
                    raise
            else:
                break

        return resp



#
# Cognica
#
# Copyright (c) 2023 Cognica, Inc.
#

# pylint: disable=no-member,missing-class-docstring,missing-function-docstring
# pylint: disable=invalid-name,no-else-return

import io
import json
import time
import typing as t

import grpc

import pandas as pd
import polars as pl
import pyarrow as pa
import pyarrow.parquet as pq

from cognica.channel import Channel
from cognica.protobuf import document_pb2, document_db_pb2, document_db_pb2_grpc


messages: t.TypeAlias = document_db_pb2  # type: ignore

DocumentDBServiceStub = document_db_pb2_grpc.DocumentDBServiceStub

IndexType: t.TypeAlias = messages.IndexType  # type: ignore
IndexStatus: t.TypeAlias = messages.IndexStatus  # type: ignore
IndexDescriptor: t.TypeAlias = messages.IndexDescriptor  # type: ignore
CreateIndexRequest: t.TypeAlias = messages.CreateIndexRequest  # type: ignore
CreateIndexResponse: t.TypeAlias = messages.CreateIndexResponse  # type: ignore
DropIndexRequest: t.TypeAlias = messages.DropIndexRequest  # type: ignore
DropIndexResponse: t.TypeAlias = messages.DropIndexResponse  # type: ignore
GetIndexRequest: t.TypeAlias = messages.GetIndexRequest  # type: ignore
GetIndexResponse: t.TypeAlias = messages.GetIndexResponse  # type: ignore

Query: t.TypeAlias = messages.Query  # type: ignore

FindRequest: t.TypeAlias = messages.FindRequest  # type: ignore
FindResponse: t.TypeAlias = messages.FindResponse  # type: ignore
FindBatchRequest: t.TypeAlias = messages.FindBatchRequest  # type: ignore
FindBatchResponse: t.TypeAlias = messages.FindBatchResponse  # type: ignore

CountRequest: t.TypeAlias = messages.CountRequest  # type: ignore
CountResponse: t.TypeAlias = messages.CountResponse  # type: ignore
ContainsRequest: t.TypeAlias = messages.ContainsRequest  # type: ignore
ContainsResponse: t.TypeAlias = messages.ContainsResponse  # type: ignore

InsertRequest: t.TypeAlias = messages.InsertRequest  # type: ignore
InsertResponse: t.TypeAlias = messages.InsertResponse  # type: ignore
UpdateRequest: t.TypeAlias = messages.UpdateRequest  # type: ignore
UpdateResponse: t.TypeAlias = messages.UpdateResponse  # type: ignore
RemoveRequest: t.TypeAlias = messages.RemoveRequest  # type: ignore
RemoveResponse: t.TypeAlias = messages.RemoveResponse  # type: ignore
GetCollectionRequest: t.TypeAlias = messages.GetCollectionRequest  # type: ignore
GetCollectionResponse: t.TypeAlias = messages.GetCollectionResponse  # type: ignore
TruncateCollectionRequest: t.TypeAlias = (
    messages.TruncateCollectionRequest  # type: ignore
)
TruncateCollectionResponse: t.TypeAlias = (
    messages.TruncateCollectionResponse  # type: ignore
)
ListCollectionsRequest: t.TypeAlias = messages.ListCollectionsRequest  # type: ignore
ListCollectionsResponse: t.TypeAlias = messages.ListCollectionsResponse  # type: ignore


def _to_json(doc):
    if isinstance(doc, document_pb2.Document):  # type: ignore
        return doc
    elif isinstance(doc, (dict, list)):
        return document_pb2.Document(  # type: ignore
            json=json.dumps(doc, ensure_ascii=False)
        )

    return document_pb2.Document(json=doc)  # type: ignore


class Request:
    def __init__(
        self,
        collection,
        query,
        index_columns=None,
        columns=None,
        dtypes=None,
        skip=None,
        limit=None,
    ):
        query = _to_json(query)

        self.collection = collection
        self.query = query
        self.index_columns = index_columns
        self.columns = columns
        self.dtypes = dtypes
        self.skip = skip
        self.limit = limit


def _create_stub(stub: t.Callable, channel: Channel):
    stub = stub(channel.channel)

    return stub


class DocumentDB:
    Request = Request

    def __init__(self, channel: Channel, timeout: int | None = None):
        self._max_retry_count = 5
        self._stub = _create_stub(DocumentDBServiceStub, channel)
        self._timeout = timeout

    def find(
        self,
        collection,
        query,
        limit=None,
        index_columns=None,
        columns=None,
        dtypes=None,
        to_pandas=True,
        to_polars=False,
        to_arrow=False,
        enable_profile=False,
    ) -> (
        pd.DataFrame
        | pl.DataFrame
        | pa.Table
        | tuple[pd.DataFrame | pl.DataFrame | pa.Table, dict]
    ):
        if to_polars or to_arrow:
            to_pandas = False

        query = _to_json(query)
        req = FindRequest(
            query=Query(collection_name=collection, query=query),
            limit=limit,
            index_columns=index_columns,
            columns=columns,
            dtypes=dtypes,
        )

        resp: FindResponse = self._invoke(
            self._stub.find, req, wait_for_ready=True
        )
        if to_pandas:
            df = self._to_pd_dataframe(
                buffer=resp.buffer,
                index_columns=index_columns,
                columns=columns,
                dtypes=dtypes,
            )
        elif to_polars:
            df = self._to_pl_dataframe(buffer=resp.buffer, columns=columns)
        elif to_arrow:
            df = self._to_arrow_table(buffer=resp.buffer, columns=columns)
        else:
            df = self._to_arrow_table(buffer=resp.buffer, columns=columns)

        if enable_profile:
            profile = {
                "duration": {
                    "query": resp.profile.query_duration_us,
                    "serialization": resp.profile.serialization_duration_us,
                    "unit": "us",
                }
            }
            del resp
            return df, profile
        else:
            del resp
            return df

    def find_batch(
        self,
        requests,
        to_pandas=True,
        to_polars=False,
        to_arrow=False,
        enable_profile=False,
    ) -> (
        list[pd.DataFrame]
        | list[pa.Table]
        | tuple[list[pd.DataFrame] | list[pa.Table], list[dict]]
    ):
        if not requests:
            return []

        if to_polars or to_arrow:
            to_pandas = False

        batch_items = []
        for req in requests:
            query = _to_json(req.query)
            item = FindRequest(
                query=Query(collection_name=req.collection, query=query),
                limit=req.limit,
                index_columns=req.index_columns,
                columns=req.columns,
                dtypes=req.dtypes,
            )
            batch_items.append(item)
        batch_req = FindBatchRequest(requests=batch_items)

        dfs = []
        profiles = []
        resp: FindBatchResponse = self._invoke(
            self._stub.find_batch, batch_req, wait_for_ready=True
        )
        for req, response in zip(requests, resp.responses):  # type: ignore
            if to_pandas:
                df = self._to_pd_dataframe(
                    buffer=response.buffer,
                    index_columns=req.index_columns,
                    columns=req.columns,
                    dtypes=req.dtypes,
                )
            elif to_polars:
                df = self._to_pl_dataframe(
                    buffer=response.buffer, columns=req.columns
                )
            elif to_arrow:
                df = self._to_arrow_table(
                    buffer=response.buffer, columns=req.columns
                )
            else:
                df = self._to_arrow_table(
                    buffer=response.buffer, columns=req.columns
                )
            dfs.append(df)
            if enable_profile:
                profile = {
                    "duration": {
                        "query": response.profile.query_duration_us,
                        "serialization": (
                            response.profile.serialization_duration_us
                        ),
                        "unit": "us",
                    }
                }
                profiles.append(profile)
        del resp

        if enable_profile:
            return dfs, profiles
        else:
            return dfs

    def insert(self, collection, docs) -> None:
        if not isinstance(docs, list):
            docs = [docs]

        items = []
        for doc in docs:
            doc = _to_json(doc)
            item = Query(collection_name=collection, query=doc)
            items.append(item)

        req = InsertRequest(requests=items)
        self._invoke(self._stub.insert, req, wait_for_ready=True)

    def update(self, collection, filter_, updates) -> None:
        filter_ = _to_json(filter_)
        updates = _to_json(updates)
        req = UpdateRequest(
            collection_name=collection, filter=filter_, updates=updates
        )

        self._invoke(self._stub.update, req, wait_for_ready=True)

    def remove(self, collection, docs) -> None:
        if not isinstance(docs, list):
            docs = [docs]

        items = []
        for doc in docs:
            doc = _to_json(doc)
            item = Query(collection_name=collection, query=doc)
            items.append(item)

        req = RemoveRequest(requests=items)

        self._invoke(self._stub.remove, req, wait_for_ready=True)

    def get_collection(self, collection) -> t.Dict[str, t.Any]:
        req = GetCollectionRequest(collection_name=collection)
        resp: GetCollectionResponse = self._invoke(
            self._stub.get_collection, req, wait_for_ready=True
        )

        return {
            "success": resp.status == 0,
            "message": resp.message,
            "data": [
                {
                    "collection_name": resp.collection.collection_name,
                    "index_descriptors": [
                        {
                            "index_id": index_desc.index_id,
                            "index_name": index_desc.index_name,
                            "fields": list(index_desc.fields),
                            "unique": index_desc.unique,
                            "index_type": IndexType.Name(index_desc.index_type),
                            "status": IndexStatus.Name(index_desc.status),
                            "options": json.loads(
                                index_desc.options.json or "{}"
                            ),
                        }
                        for index_desc in resp.collection.index_descriptors
                    ],
                    "index_stats": [
                        {
                            "index_id": index_stats.index_id,
                            "index_name": index_stats.index_name,
                            "approximated_size": index_stats.approximated_size,
                            "num_docs": index_stats.num_docs,
                            "accessed": index_stats.accessed,
                            "added": index_stats.added,
                            "updated": index_stats.updated,
                            "deleted": index_stats.deleted,
                            "merged": index_stats.merged,
                            "accessed_at": index_stats.accessed_at,
                            "added_at": index_stats.added_at,
                            "updated_at": index_stats.updated_at,
                            "deleted_at": index_stats.deleted_at,
                            "merged_at": index_stats.merged_at,
                        }
                        for index_stats in resp.collection.index_stats
                    ],
                }
            ],
        }

    def list_collections(self) -> t.List[str]:
        req = ListCollectionsRequest()

        collection_names = []
        resp: ListCollectionsResponse = self._invoke(
            self._stub.list_collections, req, wait_for_ready=True
        )
        for name in resp.collection_names:
            collection_names.append(name)

        return collection_names

    def truncate_collection(self, collection) -> None:
        req = TruncateCollectionRequest(collection_name=collection)
        self._invoke(self._stub.truncate_collection, req, wait_for_ready=True)

    def create_index(
        self, collection, index_name, fields, unique, index_type, options
    ) -> None:
        if options is not None:
            options = _to_json(options)

        index_desc = IndexDescriptor(
            index_id=0,
            index_name=index_name,
            fields=fields,
            unique=unique,
            index_type=index_type,
            status=IndexStatus.kEnabled,
            options=options,
        )
        req = CreateIndexRequest(
            collection_name=collection, index_desc=index_desc
        )
        self._invoke(self._stub.create_index, req, wait_for_ready=True)

    def drop_index(self, collection, index_name) -> None:
        req = DropIndexRequest(
            collection_name=collection, index_name=index_name
        )
        self._invoke(self._stub.drop_index, req, wait_for_ready=True)

    def get_index(self, collection, index_name) -> t.Dict[str, t.Any]:
        req = GetIndexRequest(collection_name=collection, index_name=index_name)
        resp: GetIndexResponse = self._invoke(
            self._stub.get_index, req, wait_for_ready=True
        )
        index_desc = resp.index_desc
        index_stats = resp.index_stats

        return {
            "success": resp.status == 0,
            "message": resp.message,
            "data": [
                {
                    "collection_name": resp.collection_name,
                    "index_descriptor": {
                        "index_id": index_desc.index_id,
                        "index_name": index_desc.index_name,
                        "fields": list(index_desc.fields),
                        "unique": index_desc.unique,
                        "index_type": IndexType.Name(index_desc.index_type),
                        "status": IndexStatus.Name(index_desc.status),
                        "options": json.loads(index_desc.options.json or "{}"),
                    },
                    "index_stats": {
                        "index_id": index_stats.index_id,
                        "index_name": index_stats.index_name,
                        "approximated_size": index_stats.approximated_size,
                        "num_docs": index_stats.num_docs,
                        "accessed": index_stats.accessed,
                        "added": index_stats.added,
                        "updated": index_stats.updated,
                        "deleted": index_stats.deleted,
                        "merged": index_stats.merged,
                        "accessed_at": index_stats.accessed_at,
                        "added_at": index_stats.added_at,
                        "updated_at": index_stats.updated_at,
                        "deleted_at": index_stats.deleted_at,
                        "merged_at": index_stats.merged_at,
                    },
                }
            ],
        }

    def empty(self, collection, query, dtypes=None) -> bool:
        df = self.find(collection, query, dtypes=dtypes, limit=1)

        return len(df) == 0

    def _invoke(self, func, *args, **kwargs):
        retry = 0
        backoff = 0.1
        resp = None
        if self._timeout:
            kwargs["timeout"] = self._timeout
        while retry < self._max_retry_count:
            try:
                resp = func(*args, **kwargs)
            except grpc.RpcError as rpc_error:
                if rpc_error.code() == grpc.StatusCode.UNAVAILABLE:  # type: ignore
                    retry += 1
                    time.sleep(backoff)
                    backoff *= 2
                else:
                    raise
            else:
                break

        return resp

    def _to_pd_dataframe(
        self, buffer, index_columns=None, columns=None, dtypes=None
    ) -> pd.DataFrame:
        if len(buffer) > 0:
            table = self._to_arrow_table(buffer)
            df = table.to_pandas(split_blocks=True, self_destruct=True)
            del table
        else:
            df = pd.DataFrame(columns=columns)
            if index_columns:
                try:
                    df = df.set_index(index_columns)
                except KeyError:
                    pass

        if dtypes:
            for column, type_ in dtypes.items():
                if type_ == "json" and column in df:
                    df[column] = df[column].apply(json.loads)

        return df

    def _to_pl_dataframe(
        self, buffer, columns=None, dtypes=None
    ) -> pl.DataFrame:
        if len(buffer) > 0:
            df = pl.read_parquet(io.BytesIO(buffer), columns=columns)
        else:
            df = pl.DataFrame(schema=columns)

        if dtypes:
            for column, type_ in dtypes.items():
                if type_ == "json" and column in df:
                    df[column] = df[column].apply(json.loads)

        return df

    def _to_arrow_table(self, buffer, columns=None) -> pa.Table:
        buffer_arrow = pa.py_buffer(buffer)
        table = pq.read_table(buffer_arrow, columns=columns)

        return table



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: sentence_transformer.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1asentence_transformer.proto\x12 cognica.rpc.sentence_transformer\"$\n\x06Tensor\x12\x0c\n\x04\x64ims\x18\x01 \x01(\x05\x12\x0c\n\x04\x64\x61ta\x18\x02 \x03(\x02\"\"\n\x0bProfileInfo\x12\x13\n\x0b\x64uration_us\x18\x01 \x01(\x04\"?\n\x16SentenceEncoderRequest\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x11\n\tsentences\x18\x02 \x03(\t\"\xe7\x01\n\x17SentenceEncoderResponse\x12<\n\x06status\x18\x01 \x01(\x0e\x32,.cognica.rpc.sentence_transformer.StatusType\x12\x12\n\nmodel_name\x18\x02 \x01(\t\x12\x39\n\x07tensors\x18\x03 \x03(\x0b\x32(.cognica.rpc.sentence_transformer.Tensor\x12?\n\x08profiles\x18\x04 \x03(\x0b\x32-.cognica.rpc.sentence_transformer.ProfileInfo\"4\n\x0cSentencePair\x12\x11\n\tsentence1\x18\x01 \x01(\t\x12\x11\n\tsentence2\x18\x02 \x01(\t\"l\n\x13\x43rossEncoderRequest\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x41\n\tsentences\x18\x02 \x03(\x0b\x32..cognica.rpc.sentence_transformer.SentencePair\"\xb9\x01\n\x14\x43rossEncoderResponse\x12<\n\x06status\x18\x01 \x01(\x0e\x32,.cognica.rpc.sentence_transformer.StatusType\x12\x12\n\nmodel_name\x18\x02 \x01(\t\x12\x0e\n\x06scores\x18\x03 \x03(\x02\x12?\n\x08profiles\x18\x04 \x03(\x0b\x32-.cognica.rpc.sentence_transformer.ProfileInfo\"\xb0\x01\n\x12\x43LIPEncoderRequest\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12P\n\x06\x66ormat\x18\x02 \x01(\x0e\x32@.cognica.rpc.sentence_transformer.CLIPEncoderRequest.InputFormat\x12\x0e\n\x06inputs\x18\x03 \x03(\x0c\"$\n\x0bInputFormat\x12\t\n\x05kText\x10\x00\x12\n\n\x06kImage\x10\x01\"\xe3\x01\n\x13\x43LIPEncoderResponse\x12<\n\x06status\x18\x01 \x01(\x0e\x32,.cognica.rpc.sentence_transformer.StatusType\x12\x12\n\nmodel_name\x18\x02 \x01(\t\x12\x39\n\x07tensors\x18\x03 \x03(\x0b\x32(.cognica.rpc.sentence_transformer.Tensor\x12?\n\x08profiles\x18\x04 \x03(\x0b\x32-.cognica.rpc.sentence_transformer.ProfileInfo\"Z\n\x10QAEncoderRequest\x12\x12\n\nmodel_name\x18\x01 \x01(\t\x12\x11\n\tquestions\x18\x02 \x03(\t\x12\x10\n\x08\x63ontexts\x18\x03 \x03(\t\x12\r\n\x05top_k\x18\x04 \x01(\x05\"\xa6\x03\n\x11QAEncoderResponse\x12<\n\x06status\x18\x01 \x01(\x0e\x32,.cognica.rpc.sentence_transformer.StatusType\x12\x12\n\nmodel_name\x18\x02 \x01(\t\x12R\n\x07\x61nswers\x18\x03 \x03(\x0b\x32\x41.cognica.rpc.sentence_transformer.QAEncoderResponse.CandidateList\x12?\n\x08profiles\x18\x04 \x03(\x0b\x32-.cognica.rpc.sentence_transformer.ProfileInfo\x1a\x46\n\tCandidate\x12\r\n\x05score\x18\x01 \x01(\x02\x12\r\n\x05\x62\x65gin\x18\x02 \x01(\x05\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x05\x12\x0e\n\x06\x61nswer\x18\x04 \x01(\t\x1a\x62\n\rCandidateList\x12Q\n\ncandidates\x18\x01 \x03(\x0b\x32=.cognica.rpc.sentence_transformer.QAEncoderResponse.Candidate*3\n\nStatusType\x12\x07\n\x03kOK\x10\x00\x12\r\n\tkNotFound\x10\x01\x12\r\n\tkInternal\x10\n2\x9d\x01\n\x1aSentenceTransformerService\x12\x7f\n\x06\x65ncode\x12\x38.cognica.rpc.sentence_transformer.SentenceEncoderRequest\x1a\x39.cognica.rpc.sentence_transformer.SentenceEncoderResponse\"\x00\x32\x91\x01\n\x13\x43rossEncoderService\x12z\n\x07predict\x12\x35.cognica.rpc.sentence_transformer.CrossEncoderRequest\x1a\x36.cognica.rpc.sentence_transformer.CrossEncoderResponse\"\x00\x32\x8d\x01\n\x12\x43LIPEncoderService\x12w\n\x06\x65ncode\x12\x34.cognica.rpc.sentence_transformer.CLIPEncoderRequest\x1a\x35.cognica.rpc.sentence_transformer.CLIPEncoderResponse\"\x00\x32\x88\x01\n\x10QAEncoderService\x12t\n\x07predict\x12\x32.cognica.rpc.sentence_transformer.QAEncoderRequest\x1a\x33.cognica.rpc.sentence_transformer.QAEncoderResponse\"\x00\x42\x03\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'sentence_transformer_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\370\001\001'
  _STATUSTYPE._serialized_start=1715
  _STATUSTYPE._serialized_end=1766
  _TENSOR._serialized_start=64
  _TENSOR._serialized_end=100
  _PROFILEINFO._serialized_start=102
  _PROFILEINFO._serialized_end=136
  _SENTENCEENCODERREQUEST._serialized_start=138
  _SENTENCEENCODERREQUEST._serialized_end=201
  _SENTENCEENCODERRESPONSE._serialized_start=204
  _SENTENCEENCODERRESPONSE._serialized_end=435
  _SENTENCEPAIR._serialized_start=437
  _SENTENCEPAIR._serialized_end=489
  _CROSSENCODERREQUEST._serialized_start=491
  _CROSSENCODERREQUEST._serialized_end=599
  _CROSSENCODERRESPONSE._serialized_start=602
  _CROSSENCODERRESPONSE._serialized_end=787
  _CLIPENCODERREQUEST._serialized_start=790
  _CLIPENCODERREQUEST._serialized_end=966
  _CLIPENCODERREQUEST_INPUTFORMAT._serialized_start=930
  _CLIPENCODERREQUEST_INPUTFORMAT._serialized_end=966
  _CLIPENCODERRESPONSE._serialized_start=969
  _CLIPENCODERRESPONSE._serialized_end=1196
  _QAENCODERREQUEST._serialized_start=1198
  _QAENCODERREQUEST._serialized_end=1288
  _QAENCODERRESPONSE._serialized_start=1291
  _QAENCODERRESPONSE._serialized_end=1713
  _QAENCODERRESPONSE_CANDIDATE._serialized_start=1543
  _QAENCODERRESPONSE_CANDIDATE._serialized_end=1613
  _QAENCODERRESPONSE_CANDIDATELIST._serialized_start=1615
  _QAENCODERRESPONSE_CANDIDATELIST._serialized_end=1713
  _SENTENCETRANSFORMERSERVICE._serialized_start=1769
  _SENTENCETRANSFORMERSERVICE._serialized_end=1926
  _CROSSENCODERSERVICE._serialized_start=1929
  _CROSSENCODERSERVICE._serialized_end=2074
  _CLIPENCODERSERVICE._serialized_start=2077
  _CLIPENCODERSERVICE._serialized_end=2218
  _QAENCODERSERVICE._serialized_start=2221
  _QAENCODERSERVICE._serialized_end=2357
# @@protoc_insertion_point(module_scope)



#
# Cognica
#
# Copyright (c) 2023 Cognica, Inc.
#

# pylint: disable=no-member,missing-class-docstring,missing-function-docstring

import time
import typing as t

import grpc

from cognica.channel import Channel
from cognica.protobuf import (
    fts_analysis_pipeline_pb2,
    fts_analysis_pipeline_pb2_grpc,
)


messages: t.TypeAlias = fts_analysis_pipeline_pb2  # type: ignore

FTSAnalysisPipelineServiceStub = (
    fts_analysis_pipeline_pb2_grpc.FTSAnalysisPipelineServiceStub
)

PipelineExecutionRequest: t.TypeAlias = (
    messages.PipelineExecutionRequest  # type: ignore
)
PipelineExecutionResponse: t.TypeAlias = (
    messages.PipelineExecutionResponse  # type: ignore
)
AdhocPipelineExecutionRequest: t.TypeAlias = (
    messages.AdhocPipelineExecutionRequest  # type: ignore
)
AdhocPipelineExecutionResponse: t.TypeAlias = (
    messages.AdhocPipelineExecutionResponse  # type: ignore
)


def _create_stub(stub: t.Callable, channel: Channel):
    stub = stub(channel.channel)

    return stub


class FTSAnalysisPipeline:
    def __init__(self, channel: Channel, timeout: int | None = None):
        self._max_retry_count = 5
        self._stub = _create_stub(FTSAnalysisPipelineServiceStub, channel)
        self._timeout = timeout

    def execute(
        self,
        collection_name: str,
        index_name: str,
        query: str,
        field_names: t.Optional[t.List[str]] = None,
    ) -> str:
        req = PipelineExecutionRequest(
            collection_name=collection_name,
            index_name=index_name,
            field_names=field_names,
            query=query,
        )
        resp: PipelineExecutionResponse = self._invoke(
            self._stub.execute, req, wait_for_ready=True
        )

        return resp.result

    def execute_adhoc(
        self,
        pipeline_def: str,
        query: str,
        field_names: t.Optional[t.List[str]] = None,
    ) -> str:
        req = AdhocPipelineExecutionRequest(
            pipeline_def=pipeline_def, field_names=field_names, query=query
        )
        resp: AdhocPipelineExecutionResponse = self._invoke(
            self._stub.execute_adhoc, req, wait_for_ready=True
        )

        return resp.result

    def _invoke(self, func, *args, **kwargs):
        retry = 0
        backoff = 0.1
        resp = None
        if self._timeout:
            kwargs["timeout"] = self._timeout
        while retry < self._max_retry_count:
            try:
                resp = func(*args, **kwargs)
            except grpc.RpcError as rpc_error:
                if rpc_error.code() == grpc.StatusCode.UNAVAILABLE:  # type: ignore
                    retry += 1
                    time.sleep(backoff)
                    backoff *= 2
                else:
                    raise
            else:
                break

        return resp



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import cognica.protobuf.key_value_db_pb2 as key__value__db__pb2


class KeyValueDBServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.put = channel.unary_unary(
                '/cognica.rpc.db.kv.KeyValueDBService/put',
                request_serializer=key__value__db__pb2.PutRequest.SerializeToString,
                response_deserializer=key__value__db__pb2.PutResponse.FromString,
                )
        self.remove = channel.unary_unary(
                '/cognica.rpc.db.kv.KeyValueDBService/remove',
                request_serializer=key__value__db__pb2.RemoveRequest.SerializeToString,
                response_deserializer=key__value__db__pb2.RemoveResponse.FromString,
                )
        self.get = channel.unary_unary(
                '/cognica.rpc.db.kv.KeyValueDBService/get',
                request_serializer=key__value__db__pb2.GetRequest.SerializeToString,
                response_deserializer=key__value__db__pb2.GetResponse.FromString,
                )
        self.mget = channel.unary_unary(
                '/cognica.rpc.db.kv.KeyValueDBService/mget',
                request_serializer=key__value__db__pb2.MultiGetRequest.SerializeToString,
                response_deserializer=key__value__db__pb2.MultiGetResponse.FromString,
                )
        self.put_batch = channel.unary_unary(
                '/cognica.rpc.db.kv.KeyValueDBService/put_batch',
                request_serializer=key__value__db__pb2.BatchedPutRequest.SerializeToString,
                response_deserializer=key__value__db__pb2.BatchedPutResponse.FromString,
                )
        self.remove_batch = channel.unary_unary(
                '/cognica.rpc.db.kv.KeyValueDBService/remove_batch',
                request_serializer=key__value__db__pb2.BatchedRemoveRequest.SerializeToString,
                response_deserializer=key__value__db__pb2.BatchedRemoveResponse.FromString,
                )
        self.get_batch = channel.unary_unary(
                '/cognica.rpc.db.kv.KeyValueDBService/get_batch',
                request_serializer=key__value__db__pb2.BatchedGetRequest.SerializeToString,
                response_deserializer=key__value__db__pb2.BatchedGetResponse.FromString,
                )


class KeyValueDBServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def put(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def remove(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def get(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def mget(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def put_batch(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def remove_batch(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def get_batch(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_KeyValueDBServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'put': grpc.unary_unary_rpc_method_handler(
                    servicer.put,
                    request_deserializer=key__value__db__pb2.PutRequest.FromString,
                    response_serializer=key__value__db__pb2.PutResponse.SerializeToString,
            ),
            'remove': grpc.unary_unary_rpc_method_handler(
                    servicer.remove,
                    request_deserializer=key__value__db__pb2.RemoveRequest.FromString,
                    response_serializer=key__value__db__pb2.RemoveResponse.SerializeToString,
            ),
            'get': grpc.unary_unary_rpc_method_handler(
                    servicer.get,
                    request_deserializer=key__value__db__pb2.GetRequest.FromString,
                    response_serializer=key__value__db__pb2.GetResponse.SerializeToString,
            ),
            'mget': grpc.unary_unary_rpc_method_handler(
                    servicer.mget,
                    request_deserializer=key__value__db__pb2.MultiGetRequest.FromString,
                    response_serializer=key__value__db__pb2.MultiGetResponse.SerializeToString,
            ),
            'put_batch': grpc.unary_unary_rpc_method_handler(
                    servicer.put_batch,
                    request_deserializer=key__value__db__pb2.BatchedPutRequest.FromString,
                    response_serializer=key__value__db__pb2.BatchedPutResponse.SerializeToString,
            ),
            'remove_batch': grpc.unary_unary_rpc_method_handler(
                    servicer.remove_batch,
                    request_deserializer=key__value__db__pb2.BatchedRemoveRequest.FromString,
                    response_serializer=key__value__db__pb2.BatchedRemoveResponse.SerializeToString,
            ),
            'get_batch': grpc.unary_unary_rpc_method_handler(
                    servicer.get_batch,
                    request_deserializer=key__value__db__pb2.BatchedGetRequest.FromString,
                    response_serializer=key__value__db__pb2.BatchedGetResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cognica.rpc.db.kv.KeyValueDBService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class KeyValueDBService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def put(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.kv.KeyValueDBService/put',
            key__value__db__pb2.PutRequest.SerializeToString,
            key__value__db__pb2.PutResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def remove(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.kv.KeyValueDBService/remove',
            key__value__db__pb2.RemoveRequest.SerializeToString,
            key__value__db__pb2.RemoveResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def get(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.kv.KeyValueDBService/get',
            key__value__db__pb2.GetRequest.SerializeToString,
            key__value__db__pb2.GetResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def mget(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.kv.KeyValueDBService/mget',
            key__value__db__pb2.MultiGetRequest.SerializeToString,
            key__value__db__pb2.MultiGetResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def put_batch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.kv.KeyValueDBService/put_batch',
            key__value__db__pb2.BatchedPutRequest.SerializeToString,
            key__value__db__pb2.BatchedPutResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def remove_batch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.kv.KeyValueDBService/remove_batch',
            key__value__db__pb2.BatchedRemoveRequest.SerializeToString,
            key__value__db__pb2.BatchedRemoveResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def get_batch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.kv.KeyValueDBService/get_batch',
            key__value__db__pb2.BatchedGetRequest.SerializeToString,
            key__value__db__pb2.BatchedGetResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class KeyspaceManagerServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.create_keyspace = channel.unary_unary(
                '/cognica.rpc.db.kv.KeyspaceManagerService/create_keyspace',
                request_serializer=key__value__db__pb2.CreateKeyspaceRequest.SerializeToString,
                response_deserializer=key__value__db__pb2.CreateKeyspaceResponse.FromString,
                )
        self.drop_keyspace = channel.unary_unary(
                '/cognica.rpc.db.kv.KeyspaceManagerService/drop_keyspace',
                request_serializer=key__value__db__pb2.DropKeyspaceRequest.SerializeToString,
                response_deserializer=key__value__db__pb2.DropKeyspaceResponse.FromString,
                )
        self.truncate_keyspace = channel.unary_unary(
                '/cognica.rpc.db.kv.KeyspaceManagerService/truncate_keyspace',
                request_serializer=key__value__db__pb2.TruncateKeyspaceRequest.SerializeToString,
                response_deserializer=key__value__db__pb2.TruncateKeyspaceResponse.FromString,
                )
        self.list_keyspaces = channel.unary_unary(
                '/cognica.rpc.db.kv.KeyspaceManagerService/list_keyspaces',
                request_serializer=key__value__db__pb2.ListKeyspacesRequest.SerializeToString,
                response_deserializer=key__value__db__pb2.ListKeyspacesResponse.FromString,
                )


class KeyspaceManagerServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def create_keyspace(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def drop_keyspace(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def truncate_keyspace(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def list_keyspaces(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_KeyspaceManagerServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'create_keyspace': grpc.unary_unary_rpc_method_handler(
                    servicer.create_keyspace,
                    request_deserializer=key__value__db__pb2.CreateKeyspaceRequest.FromString,
                    response_serializer=key__value__db__pb2.CreateKeyspaceResponse.SerializeToString,
            ),
            'drop_keyspace': grpc.unary_unary_rpc_method_handler(
                    servicer.drop_keyspace,
                    request_deserializer=key__value__db__pb2.DropKeyspaceRequest.FromString,
                    response_serializer=key__value__db__pb2.DropKeyspaceResponse.SerializeToString,
            ),
            'truncate_keyspace': grpc.unary_unary_rpc_method_handler(
                    servicer.truncate_keyspace,
                    request_deserializer=key__value__db__pb2.TruncateKeyspaceRequest.FromString,
                    response_serializer=key__value__db__pb2.TruncateKeyspaceResponse.SerializeToString,
            ),
            'list_keyspaces': grpc.unary_unary_rpc_method_handler(
                    servicer.list_keyspaces,
                    request_deserializer=key__value__db__pb2.ListKeyspacesRequest.FromString,
                    response_serializer=key__value__db__pb2.ListKeyspacesResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cognica.rpc.db.kv.KeyspaceManagerService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class KeyspaceManagerService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def create_keyspace(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.kv.KeyspaceManagerService/create_keyspace',
            key__value__db__pb2.CreateKeyspaceRequest.SerializeToString,
            key__value__db__pb2.CreateKeyspaceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def drop_keyspace(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.kv.KeyspaceManagerService/drop_keyspace',
            key__value__db__pb2.DropKeyspaceRequest.SerializeToString,
            key__value__db__pb2.DropKeyspaceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def truncate_keyspace(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.kv.KeyspaceManagerService/truncate_keyspace',
            key__value__db__pb2.TruncateKeyspaceRequest.SerializeToString,
            key__value__db__pb2.TruncateKeyspaceResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def list_keyspaces(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.kv.KeyspaceManagerService/list_keyspaces',
            key__value__db__pb2.ListKeyspacesRequest.SerializeToString,
            key__value__db__pb2.ListKeyspacesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import cognica.protobuf.sentence_transformer_pb2 as sentence__transformer__pb2


class SentenceTransformerServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.encode = channel.unary_unary(
                '/cognica.rpc.sentence_transformer.SentenceTransformerService/encode',
                request_serializer=sentence__transformer__pb2.SentenceEncoderRequest.SerializeToString,
                response_deserializer=sentence__transformer__pb2.SentenceEncoderResponse.FromString,
                )


class SentenceTransformerServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def encode(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SentenceTransformerServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'encode': grpc.unary_unary_rpc_method_handler(
                    servicer.encode,
                    request_deserializer=sentence__transformer__pb2.SentenceEncoderRequest.FromString,
                    response_serializer=sentence__transformer__pb2.SentenceEncoderResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cognica.rpc.sentence_transformer.SentenceTransformerService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class SentenceTransformerService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def encode(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.sentence_transformer.SentenceTransformerService/encode',
            sentence__transformer__pb2.SentenceEncoderRequest.SerializeToString,
            sentence__transformer__pb2.SentenceEncoderResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class CrossEncoderServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.predict = channel.unary_unary(
                '/cognica.rpc.sentence_transformer.CrossEncoderService/predict',
                request_serializer=sentence__transformer__pb2.CrossEncoderRequest.SerializeToString,
                response_deserializer=sentence__transformer__pb2.CrossEncoderResponse.FromString,
                )


class CrossEncoderServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def predict(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_CrossEncoderServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'predict': grpc.unary_unary_rpc_method_handler(
                    servicer.predict,
                    request_deserializer=sentence__transformer__pb2.CrossEncoderRequest.FromString,
                    response_serializer=sentence__transformer__pb2.CrossEncoderResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cognica.rpc.sentence_transformer.CrossEncoderService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class CrossEncoderService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def predict(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.sentence_transformer.CrossEncoderService/predict',
            sentence__transformer__pb2.CrossEncoderRequest.SerializeToString,
            sentence__transformer__pb2.CrossEncoderResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class CLIPEncoderServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.encode = channel.unary_unary(
                '/cognica.rpc.sentence_transformer.CLIPEncoderService/encode',
                request_serializer=sentence__transformer__pb2.CLIPEncoderRequest.SerializeToString,
                response_deserializer=sentence__transformer__pb2.CLIPEncoderResponse.FromString,
                )


class CLIPEncoderServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def encode(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_CLIPEncoderServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'encode': grpc.unary_unary_rpc_method_handler(
                    servicer.encode,
                    request_deserializer=sentence__transformer__pb2.CLIPEncoderRequest.FromString,
                    response_serializer=sentence__transformer__pb2.CLIPEncoderResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cognica.rpc.sentence_transformer.CLIPEncoderService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class CLIPEncoderService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def encode(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.sentence_transformer.CLIPEncoderService/encode',
            sentence__transformer__pb2.CLIPEncoderRequest.SerializeToString,
            sentence__transformer__pb2.CLIPEncoderResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)


class QAEncoderServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.predict = channel.unary_unary(
                '/cognica.rpc.sentence_transformer.QAEncoderService/predict',
                request_serializer=sentence__transformer__pb2.QAEncoderRequest.SerializeToString,
                response_deserializer=sentence__transformer__pb2.QAEncoderResponse.FromString,
                )


class QAEncoderServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def predict(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QAEncoderServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'predict': grpc.unary_unary_rpc_method_handler(
                    servicer.predict,
                    request_deserializer=sentence__transformer__pb2.QAEncoderRequest.FromString,
                    response_serializer=sentence__transformer__pb2.QAEncoderResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cognica.rpc.sentence_transformer.QAEncoderService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class QAEncoderService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def predict(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.sentence_transformer.QAEncoderService/predict',
            sentence__transformer__pb2.QAEncoderRequest.SerializeToString,
            sentence__transformer__pb2.QAEncoderResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: document.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x64ocument.proto\x12\x17\x63ognica.rpc.db.document\"\xdb\x01\n\x05Value\x12\x0e\n\x04null\x18\x01 \x01(\x08H\x00\x12\x0e\n\x04\x62ool\x18\x02 \x01(\x08H\x00\x12\x0f\n\x05int64\x18\x03 \x01(\x03H\x00\x12\x10\n\x06uint64\x18\x04 \x01(\x04H\x00\x12\x10\n\x06\x64ouble\x18\x05 \x01(\x01H\x00\x12\x10\n\x06string\x18\x06 \x01(\tH\x00\x12/\n\x05\x61rray\x18\x07 \x01(\x0b\x32\x1e.cognica.rpc.db.document.ArrayH\x00\x12\x31\n\x06object\x18\x08 \x01(\x0b\x32\x1f.cognica.rpc.db.document.ObjectH\x00\x42\x07\n\x05value\"6\n\x05\x41rray\x12-\n\x05value\x18\x01 \x03(\x0b\x32\x1e.cognica.rpc.db.document.Value\"\x91\x01\n\x06Object\x12\x39\n\x05value\x18\x01 \x03(\x0b\x32*.cognica.rpc.db.document.Object.ValueEntry\x1aL\n\nValueEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12-\n\x05value\x18\x02 \x01(\x0b\x32\x1e.cognica.rpc.db.document.Value:\x02\x38\x01\"U\n\x08\x44ocument\x12\x31\n\x06object\x18\x01 \x01(\x0b\x32\x1f.cognica.rpc.db.document.ObjectH\x00\x12\x0e\n\x04json\x18\x02 \x01(\tH\x00\x42\x06\n\x04typeB\x03\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'document_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\370\001\001'
  _OBJECT_VALUEENTRY._options = None
  _OBJECT_VALUEENTRY._serialized_options = b'8\001'
  _VALUE._serialized_start=44
  _VALUE._serialized_end=263
  _ARRAY._serialized_start=265
  _ARRAY._serialized_end=319
  _OBJECT._serialized_start=322
  _OBJECT._serialized_end=467
  _OBJECT_VALUEENTRY._serialized_start=391
  _OBJECT_VALUEENTRY._serialized_end=467
  _DOCUMENT._serialized_start=469
  _DOCUMENT._serialized_end=554
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import cognica.protobuf.document_db_pb2 as document__db__pb2


class DocumentDBServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.find = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/find',
                request_serializer=document__db__pb2.FindRequest.SerializeToString,
                response_deserializer=document__db__pb2.FindResponse.FromString,
                )
        self.find_batch = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/find_batch',
                request_serializer=document__db__pb2.FindBatchRequest.SerializeToString,
                response_deserializer=document__db__pb2.FindBatchResponse.FromString,
                )
        self.count = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/count',
                request_serializer=document__db__pb2.CountRequest.SerializeToString,
                response_deserializer=document__db__pb2.CountResponse.FromString,
                )
        self.contains = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/contains',
                request_serializer=document__db__pb2.ContainsRequest.SerializeToString,
                response_deserializer=document__db__pb2.ContainsResponse.FromString,
                )
        self.insert = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/insert',
                request_serializer=document__db__pb2.InsertRequest.SerializeToString,
                response_deserializer=document__db__pb2.InsertResponse.FromString,
                )
        self.update = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/update',
                request_serializer=document__db__pb2.UpdateRequest.SerializeToString,
                response_deserializer=document__db__pb2.UpdateResponse.FromString,
                )
        self.remove = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/remove',
                request_serializer=document__db__pb2.RemoveRequest.SerializeToString,
                response_deserializer=document__db__pb2.RemoveResponse.FromString,
                )
        self.create_collection = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/create_collection',
                request_serializer=document__db__pb2.CreateCollectionRequest.SerializeToString,
                response_deserializer=document__db__pb2.CreateCollectionResponse.FromString,
                )
        self.drop_collection = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/drop_collection',
                request_serializer=document__db__pb2.DropCollectionRequest.SerializeToString,
                response_deserializer=document__db__pb2.DropCollectionResponse.FromString,
                )
        self.get_collection = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/get_collection',
                request_serializer=document__db__pb2.GetCollectionRequest.SerializeToString,
                response_deserializer=document__db__pb2.GetCollectionResponse.FromString,
                )
        self.list_collections = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/list_collections',
                request_serializer=document__db__pb2.ListCollectionsRequest.SerializeToString,
                response_deserializer=document__db__pb2.ListCollectionsResponse.FromString,
                )
        self.truncate_collection = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/truncate_collection',
                request_serializer=document__db__pb2.TruncateCollectionRequest.SerializeToString,
                response_deserializer=document__db__pb2.TruncateCollectionResponse.FromString,
                )
        self.create_index = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/create_index',
                request_serializer=document__db__pb2.CreateIndexRequest.SerializeToString,
                response_deserializer=document__db__pb2.CreateIndexResponse.FromString,
                )
        self.drop_index = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/drop_index',
                request_serializer=document__db__pb2.DropIndexRequest.SerializeToString,
                response_deserializer=document__db__pb2.DropIndexResponse.FromString,
                )
        self.get_index = channel.unary_unary(
                '/cognica.rpc.db.document.DocumentDBService/get_index',
                request_serializer=document__db__pb2.GetIndexRequest.SerializeToString,
                response_deserializer=document__db__pb2.GetIndexResponse.FromString,
                )


class DocumentDBServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def find(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def find_batch(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def count(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def contains(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def insert(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def update(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def remove(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def create_collection(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def drop_collection(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def get_collection(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def list_collections(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def truncate_collection(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def create_index(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def drop_index(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def get_index(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_DocumentDBServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'find': grpc.unary_unary_rpc_method_handler(
                    servicer.find,
                    request_deserializer=document__db__pb2.FindRequest.FromString,
                    response_serializer=document__db__pb2.FindResponse.SerializeToString,
            ),
            'find_batch': grpc.unary_unary_rpc_method_handler(
                    servicer.find_batch,
                    request_deserializer=document__db__pb2.FindBatchRequest.FromString,
                    response_serializer=document__db__pb2.FindBatchResponse.SerializeToString,
            ),
            'count': grpc.unary_unary_rpc_method_handler(
                    servicer.count,
                    request_deserializer=document__db__pb2.CountRequest.FromString,
                    response_serializer=document__db__pb2.CountResponse.SerializeToString,
            ),
            'contains': grpc.unary_unary_rpc_method_handler(
                    servicer.contains,
                    request_deserializer=document__db__pb2.ContainsRequest.FromString,
                    response_serializer=document__db__pb2.ContainsResponse.SerializeToString,
            ),
            'insert': grpc.unary_unary_rpc_method_handler(
                    servicer.insert,
                    request_deserializer=document__db__pb2.InsertRequest.FromString,
                    response_serializer=document__db__pb2.InsertResponse.SerializeToString,
            ),
            'update': grpc.unary_unary_rpc_method_handler(
                    servicer.update,
                    request_deserializer=document__db__pb2.UpdateRequest.FromString,
                    response_serializer=document__db__pb2.UpdateResponse.SerializeToString,
            ),
            'remove': grpc.unary_unary_rpc_method_handler(
                    servicer.remove,
                    request_deserializer=document__db__pb2.RemoveRequest.FromString,
                    response_serializer=document__db__pb2.RemoveResponse.SerializeToString,
            ),
            'create_collection': grpc.unary_unary_rpc_method_handler(
                    servicer.create_collection,
                    request_deserializer=document__db__pb2.CreateCollectionRequest.FromString,
                    response_serializer=document__db__pb2.CreateCollectionResponse.SerializeToString,
            ),
            'drop_collection': grpc.unary_unary_rpc_method_handler(
                    servicer.drop_collection,
                    request_deserializer=document__db__pb2.DropCollectionRequest.FromString,
                    response_serializer=document__db__pb2.DropCollectionResponse.SerializeToString,
            ),
            'get_collection': grpc.unary_unary_rpc_method_handler(
                    servicer.get_collection,
                    request_deserializer=document__db__pb2.GetCollectionRequest.FromString,
                    response_serializer=document__db__pb2.GetCollectionResponse.SerializeToString,
            ),
            'list_collections': grpc.unary_unary_rpc_method_handler(
                    servicer.list_collections,
                    request_deserializer=document__db__pb2.ListCollectionsRequest.FromString,
                    response_serializer=document__db__pb2.ListCollectionsResponse.SerializeToString,
            ),
            'truncate_collection': grpc.unary_unary_rpc_method_handler(
                    servicer.truncate_collection,
                    request_deserializer=document__db__pb2.TruncateCollectionRequest.FromString,
                    response_serializer=document__db__pb2.TruncateCollectionResponse.SerializeToString,
            ),
            'create_index': grpc.unary_unary_rpc_method_handler(
                    servicer.create_index,
                    request_deserializer=document__db__pb2.CreateIndexRequest.FromString,
                    response_serializer=document__db__pb2.CreateIndexResponse.SerializeToString,
            ),
            'drop_index': grpc.unary_unary_rpc_method_handler(
                    servicer.drop_index,
                    request_deserializer=document__db__pb2.DropIndexRequest.FromString,
                    response_serializer=document__db__pb2.DropIndexResponse.SerializeToString,
            ),
            'get_index': grpc.unary_unary_rpc_method_handler(
                    servicer.get_index,
                    request_deserializer=document__db__pb2.GetIndexRequest.FromString,
                    response_serializer=document__db__pb2.GetIndexResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cognica.rpc.db.document.DocumentDBService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class DocumentDBService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def find(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/find',
            document__db__pb2.FindRequest.SerializeToString,
            document__db__pb2.FindResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def find_batch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/find_batch',
            document__db__pb2.FindBatchRequest.SerializeToString,
            document__db__pb2.FindBatchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def count(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/count',
            document__db__pb2.CountRequest.SerializeToString,
            document__db__pb2.CountResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def contains(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/contains',
            document__db__pb2.ContainsRequest.SerializeToString,
            document__db__pb2.ContainsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def insert(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/insert',
            document__db__pb2.InsertRequest.SerializeToString,
            document__db__pb2.InsertResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def update(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/update',
            document__db__pb2.UpdateRequest.SerializeToString,
            document__db__pb2.UpdateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def remove(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/remove',
            document__db__pb2.RemoveRequest.SerializeToString,
            document__db__pb2.RemoveResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def create_collection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/create_collection',
            document__db__pb2.CreateCollectionRequest.SerializeToString,
            document__db__pb2.CreateCollectionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def drop_collection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/drop_collection',
            document__db__pb2.DropCollectionRequest.SerializeToString,
            document__db__pb2.DropCollectionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def get_collection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/get_collection',
            document__db__pb2.GetCollectionRequest.SerializeToString,
            document__db__pb2.GetCollectionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def list_collections(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/list_collections',
            document__db__pb2.ListCollectionsRequest.SerializeToString,
            document__db__pb2.ListCollectionsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def truncate_collection(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/truncate_collection',
            document__db__pb2.TruncateCollectionRequest.SerializeToString,
            document__db__pb2.TruncateCollectionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def create_index(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/create_index',
            document__db__pb2.CreateIndexRequest.SerializeToString,
            document__db__pb2.CreateIndexResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def drop_index(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/drop_index',
            document__db__pb2.DropIndexRequest.SerializeToString,
            document__db__pb2.DropIndexResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def get_index(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.document.DocumentDBService/get_index',
            document__db__pb2.GetIndexRequest.SerializeToString,
            document__db__pb2.GetIndexResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: document_db.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import cognica.protobuf.document_pb2 as document__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x64ocument_db.proto\x12\x17\x63ognica.rpc.db.document\x1a\x0e\x64ocument.proto\"\xf9\x01\n\x0fIndexDescriptor\x12\x10\n\x08index_id\x18\x01 \x01(\r\x12\x12\n\nindex_name\x18\x02 \x01(\t\x12\x0e\n\x06\x66ields\x18\x03 \x03(\t\x12\x0e\n\x06unique\x18\x04 \x01(\x08\x12\x36\n\nindex_type\x18\x05 \x01(\x0e\x32\".cognica.rpc.db.document.IndexType\x12\x34\n\x06status\x18\x06 \x01(\x0e\x32$.cognica.rpc.db.document.IndexStatus\x12\x32\n\x07options\x18\x07 \x01(\x0b\x32!.cognica.rpc.db.document.Document\"\xa6\x01\n\rFTSFieldStats\x12\x12\n\nfield_name\x18\x01 \x01(\t\x12\x17\n\x0ftotal_doc_count\x18\x02 \x01(\x03\x12\x16\n\x0etotal_doc_size\x18\x03 \x01(\x03\x12\x11\n\tdoc_count\x18\x04 \x01(\x03\x12\x10\n\x08\x64oc_size\x18\x05 \x01(\x03\x12\x15\n\rsum_term_freq\x18\x06 \x01(\x03\x12\x14\n\x0csum_doc_freq\x18\x07 \x01(\x03\"q\n\rFTSIndexStats\x12\x11\n\tdoc_count\x18\x01 \x01(\x03\x12\x10\n\x08\x64oc_size\x18\x02 \x01(\x03\x12;\n\x0b\x66ield_stats\x18\x03 \x03(\x0b\x32&.cognica.rpc.db.document.FTSFieldStats\"\xcf\x02\n\nIndexStats\x12\x10\n\x08index_id\x18\x01 \x01(\r\x12\x12\n\nindex_name\x18\x02 \x01(\t\x12\x19\n\x11\x61pproximated_size\x18\x03 \x01(\x04\x12\x10\n\x08num_docs\x18\x04 \x01(\x04\x12\x10\n\x08\x61\x63\x63\x65ssed\x18\x05 \x01(\x04\x12\r\n\x05\x61\x64\x64\x65\x64\x18\x06 \x01(\x04\x12\x0f\n\x07updated\x18\x07 \x01(\x04\x12\x0f\n\x07\x64\x65leted\x18\x08 \x01(\x04\x12\x0e\n\x06merged\x18\t \x01(\x04\x12\x13\n\x0b\x61\x63\x63\x65ssed_at\x18\n \x01(\x03\x12\x10\n\x08\x61\x64\x64\x65\x64_at\x18\x0b \x01(\x03\x12\x12\n\nupdated_at\x18\x0c \x01(\x03\x12\x12\n\ndeleted_at\x18\r \x01(\x03\x12\x11\n\tmerged_at\x18\x0e \x01(\x03\x12\x39\n\tfts_stats\x18\x0f \x01(\x0b\x32&.cognica.rpc.db.document.FTSIndexStats\"\xa8\x01\n\x0e\x43ollectionInfo\x12\x17\n\x0f\x63ollection_name\x18\x01 \x01(\t\x12\x43\n\x11index_descriptors\x18\x02 \x03(\x0b\x32(.cognica.rpc.db.document.IndexDescriptor\x12\x38\n\x0bindex_stats\x18\x03 \x03(\x0b\x32#.cognica.rpc.db.document.IndexStats\"\x7f\n\x0bProfileInfo\x12\x0f\n\x07matched\x18\x01 \x01(\x04\x12\x0f\n\x07scanned\x18\x02 \x01(\x04\x12\x10\n\x08\x66iltered\x18\x03 \x01(\x04\x12\x19\n\x11query_duration_us\x18\x04 \x01(\x04\x12!\n\x19serialization_duration_us\x18\x05 \x01(\x04\"V\n\x17\x43reateCollectionRequest\x12;\n\ncollection\x18\x01 \x01(\x0b\x32\'.cognica.rpc.db.document.CollectionInfo\"r\n\x18\x43reateCollectionResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x35\n\x07profile\x18\x03 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\"0\n\x15\x44ropCollectionRequest\x12\x17\n\x0f\x63ollection_name\x18\x01 \x01(\t\"p\n\x16\x44ropCollectionResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x35\n\x07profile\x18\x03 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\"/\n\x14GetCollectionRequest\x12\x17\n\x0f\x63ollection_name\x18\x01 \x01(\t\"\xac\x01\n\x15GetCollectionResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12;\n\ncollection\x18\x03 \x01(\x0b\x32\'.cognica.rpc.db.document.CollectionInfo\x12\x35\n\x07profile\x18\x04 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\"k\n\x12\x43reateIndexRequest\x12\x17\n\x0f\x63ollection_name\x18\x01 \x01(\t\x12<\n\nindex_desc\x18\x02 \x01(\x0b\x32(.cognica.rpc.db.document.IndexDescriptor\"m\n\x13\x43reateIndexResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x35\n\x07profile\x18\x03 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\"?\n\x10\x44ropIndexRequest\x12\x17\n\x0f\x63ollection_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\"k\n\x11\x44ropIndexResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x35\n\x07profile\x18\x03 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\">\n\x0fGetIndexRequest\x12\x17\n\x0f\x63ollection_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\"\xfb\x01\n\x10GetIndexResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x17\n\x0f\x63ollection_name\x18\x03 \x01(\t\x12<\n\nindex_desc\x18\x04 \x01(\x0b\x32(.cognica.rpc.db.document.IndexDescriptor\x12\x38\n\x0bindex_stats\x18\x05 \x01(\x0b\x32#.cognica.rpc.db.document.IndexStats\x12\x35\n\x07profile\x18\x06 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\"R\n\x05Query\x12\x17\n\x0f\x63ollection_name\x18\x01 \x01(\t\x12\x30\n\x05query\x18\x02 \x01(\x0b\x32!.cognica.rpc.db.document.Document\"\xe4\x01\n\x0b\x46indRequest\x12-\n\x05query\x18\x01 \x01(\x0b\x32\x1e.cognica.rpc.db.document.Query\x12\r\n\x05limit\x18\x02 \x01(\x05\x12\x15\n\rindex_columns\x18\x03 \x03(\t\x12\x0f\n\x07\x63olumns\x18\x04 \x03(\t\x12@\n\x06\x64types\x18\x05 \x03(\x0b\x32\x30.cognica.rpc.db.document.FindRequest.DtypesEntry\x1a-\n\x0b\x44typesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"|\n\x0c\x46indResponse\x12\x13\n\x0bnum_columns\x18\x01 \x01(\x05\x12\x10\n\x08num_rows\x18\x02 \x01(\x05\x12\x0e\n\x06\x62uffer\x18\x03 \x01(\x0c\x12\x35\n\x07profile\x18\x04 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\"J\n\x10\x46indBatchRequest\x12\x36\n\x08requests\x18\x01 \x03(\x0b\x32$.cognica.rpc.db.document.FindRequest\"M\n\x11\x46indBatchResponse\x12\x38\n\tresponses\x18\x01 \x03(\x0b\x32%.cognica.rpc.db.document.FindResponse\"=\n\x0c\x43ountRequest\x12-\n\x05query\x18\x01 \x01(\x0b\x32\x1e.cognica.rpc.db.document.Query\"v\n\rCountResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\r\n\x05\x63ount\x18\x03 \x01(\x03\x12\x35\n\x07profile\x18\x04 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\"@\n\x0f\x43ontainsRequest\x12-\n\x05query\x18\x01 \x01(\x0b\x32\x1e.cognica.rpc.db.document.Query\"y\n\x10\x43ontainsResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\r\n\x05\x66ound\x18\x03 \x01(\x08\x12\x35\n\x07profile\x18\x04 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\"A\n\rInsertRequest\x12\x30\n\x08requests\x18\x01 \x03(\x0b\x32\x1e.cognica.rpc.db.document.Query\"h\n\x0eInsertResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x35\n\x07profile\x18\x03 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\"\x8f\x01\n\rUpdateRequest\x12\x17\n\x0f\x63ollection_name\x18\x01 \x01(\t\x12\x31\n\x06\x66ilter\x18\x02 \x01(\x0b\x32!.cognica.rpc.db.document.Document\x12\x32\n\x07updates\x18\x03 \x01(\x0b\x32!.cognica.rpc.db.document.Document\"h\n\x0eUpdateResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x35\n\x07profile\x18\x03 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\"A\n\rRemoveRequest\x12\x30\n\x08requests\x18\x01 \x03(\x0b\x32\x1e.cognica.rpc.db.document.Query\"h\n\x0eRemoveResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x35\n\x07profile\x18\x03 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\"4\n\x19TruncateCollectionRequest\x12\x17\n\x0f\x63ollection_name\x18\x01 \x01(\t\"t\n\x1aTruncateCollectionResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x35\n\x07profile\x18\x03 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo\"\x18\n\x16ListCollectionsRequest\"\x8b\x01\n\x17ListCollectionsResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0f\n\x07message\x18\x02 \x01(\t\x12\x18\n\x10\x63ollection_names\x18\x03 \x03(\t\x12\x35\n\x07profile\x18\x04 \x01(\x0b\x32$.cognica.rpc.db.document.ProfileInfo*e\n\tIndexType\x12\x0f\n\x0bkPrimaryKey\x10\x00\x12\x11\n\rkSecondaryKey\x10\x01\x12\x1a\n\x16kClusteredSecondaryKey\x10\x02\x12\x18\n\x14kFullTextSearchIndex\x10\x03*\x8d\x01\n\x0bIndexStatus\x12\x0c\n\x08kEnabled\x10\x00\x12\r\n\tkDisabled\x10\x01\x12\x0f\n\x0bkReadyToUse\x10\x02\x12\x14\n\x10kBuildInProgress\x10\x03\x12\x12\n\x0ekBuildFinished\x10\x04\x12\x13\n\x0fkDropInProgress\x10\x05\x12\x11\n\rkDropFinished\x10\x06\x32\xbe\x0c\n\x11\x44ocumentDBService\x12U\n\x04\x66ind\x12$.cognica.rpc.db.document.FindRequest\x1a%.cognica.rpc.db.document.FindResponse\"\x00\x12\x65\n\nfind_batch\x12).cognica.rpc.db.document.FindBatchRequest\x1a*.cognica.rpc.db.document.FindBatchResponse\"\x00\x12X\n\x05\x63ount\x12%.cognica.rpc.db.document.CountRequest\x1a&.cognica.rpc.db.document.CountResponse\"\x00\x12\x61\n\x08\x63ontains\x12(.cognica.rpc.db.document.ContainsRequest\x1a).cognica.rpc.db.document.ContainsResponse\"\x00\x12[\n\x06insert\x12&.cognica.rpc.db.document.InsertRequest\x1a\'.cognica.rpc.db.document.InsertResponse\"\x00\x12[\n\x06update\x12&.cognica.rpc.db.document.UpdateRequest\x1a\'.cognica.rpc.db.document.UpdateResponse\"\x00\x12[\n\x06remove\x12&.cognica.rpc.db.document.RemoveRequest\x1a\'.cognica.rpc.db.document.RemoveResponse\"\x00\x12z\n\x11\x63reate_collection\x12\x30.cognica.rpc.db.document.CreateCollectionRequest\x1a\x31.cognica.rpc.db.document.CreateCollectionResponse\"\x00\x12t\n\x0f\x64rop_collection\x12..cognica.rpc.db.document.DropCollectionRequest\x1a/.cognica.rpc.db.document.DropCollectionResponse\"\x00\x12q\n\x0eget_collection\x12-.cognica.rpc.db.document.GetCollectionRequest\x1a..cognica.rpc.db.document.GetCollectionResponse\"\x00\x12w\n\x10list_collections\x12/.cognica.rpc.db.document.ListCollectionsRequest\x1a\x30.cognica.rpc.db.document.ListCollectionsResponse\"\x00\x12\x80\x01\n\x13truncate_collection\x12\x32.cognica.rpc.db.document.TruncateCollectionRequest\x1a\x33.cognica.rpc.db.document.TruncateCollectionResponse\"\x00\x12k\n\x0c\x63reate_index\x12+.cognica.rpc.db.document.CreateIndexRequest\x1a,.cognica.rpc.db.document.CreateIndexResponse\"\x00\x12\x65\n\ndrop_index\x12).cognica.rpc.db.document.DropIndexRequest\x1a*.cognica.rpc.db.document.DropIndexResponse\"\x00\x12\x62\n\tget_index\x12(.cognica.rpc.db.document.GetIndexRequest\x1a).cognica.rpc.db.document.GetIndexResponse\"\x00\x42\x03\xf8\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'document_db_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\370\001\001'
  _FINDREQUEST_DTYPESENTRY._options = None
  _FINDREQUEST_DTYPESENTRY._serialized_options = b'8\001'
  _globals['_INDEXTYPE']._serialized_start=4446
  _globals['_INDEXTYPE']._serialized_end=4547
  _globals['_INDEXSTATUS']._serialized_start=4550
  _globals['_INDEXSTATUS']._serialized_end=4691
  _globals['_INDEXDESCRIPTOR']._serialized_start=63
  _globals['_INDEXDESCRIPTOR']._serialized_end=312
  _globals['_FTSFIELDSTATS']._serialized_start=315
  _globals['_FTSFIELDSTATS']._serialized_end=481
  _globals['_FTSINDEXSTATS']._serialized_start=483
  _globals['_FTSINDEXSTATS']._serialized_end=596
  _globals['_INDEXSTATS']._serialized_start=599
  _globals['_INDEXSTATS']._serialized_end=934
  _globals['_COLLECTIONINFO']._serialized_start=937
  _globals['_COLLECTIONINFO']._serialized_end=1105
  _globals['_PROFILEINFO']._serialized_start=1107
  _globals['_PROFILEINFO']._serialized_end=1234
  _globals['_CREATECOLLECTIONREQUEST']._serialized_start=1236
  _globals['_CREATECOLLECTIONREQUEST']._serialized_end=1322
  _globals['_CREATECOLLECTIONRESPONSE']._serialized_start=1324
  _globals['_CREATECOLLECTIONRESPONSE']._serialized_end=1438
  _globals['_DROPCOLLECTIONREQUEST']._serialized_start=1440
  _globals['_DROPCOLLECTIONREQUEST']._serialized_end=1488
  _globals['_DROPCOLLECTIONRESPONSE']._serialized_start=1490
  _globals['_DROPCOLLECTIONRESPONSE']._serialized_end=1602
  _globals['_GETCOLLECTIONREQUEST']._serialized_start=1604
  _globals['_GETCOLLECTIONREQUEST']._serialized_end=1651
  _globals['_GETCOLLECTIONRESPONSE']._serialized_start=1654
  _globals['_GETCOLLECTIONRESPONSE']._serialized_end=1826
  _globals['_CREATEINDEXREQUEST']._serialized_start=1828
  _globals['_CREATEINDEXREQUEST']._serialized_end=1935
  _globals['_CREATEINDEXRESPONSE']._serialized_start=1937
  _globals['_CREATEINDEXRESPONSE']._serialized_end=2046
  _globals['_DROPINDEXREQUEST']._serialized_start=2048
  _globals['_DROPINDEXREQUEST']._serialized_end=2111
  _globals['_DROPINDEXRESPONSE']._serialized_start=2113
  _globals['_DROPINDEXRESPONSE']._serialized_end=2220
  _globals['_GETINDEXREQUEST']._serialized_start=2222
  _globals['_GETINDEXREQUEST']._serialized_end=2284
  _globals['_GETINDEXRESPONSE']._serialized_start=2287
  _globals['_GETINDEXRESPONSE']._serialized_end=2538
  _globals['_QUERY']._serialized_start=2540
  _globals['_QUERY']._serialized_end=2622
  _globals['_FINDREQUEST']._serialized_start=2625
  _globals['_FINDREQUEST']._serialized_end=2853
  _globals['_FINDREQUEST_DTYPESENTRY']._serialized_start=2808
  _globals['_FINDREQUEST_DTYPESENTRY']._serialized_end=2853
  _globals['_FINDRESPONSE']._serialized_start=2855
  _globals['_FINDRESPONSE']._serialized_end=2979
  _globals['_FINDBATCHREQUEST']._serialized_start=2981
  _globals['_FINDBATCHREQUEST']._serialized_end=3055
  _globals['_FINDBATCHRESPONSE']._serialized_start=3057
  _globals['_FINDBATCHRESPONSE']._serialized_end=3134
  _globals['_COUNTREQUEST']._serialized_start=3136
  _globals['_COUNTREQUEST']._serialized_end=3197
  _globals['_COUNTRESPONSE']._serialized_start=3199
  _globals['_COUNTRESPONSE']._serialized_end=3317
  _globals['_CONTAINSREQUEST']._serialized_start=3319
  _globals['_CONTAINSREQUEST']._serialized_end=3383
  _globals['_CONTAINSRESPONSE']._serialized_start=3385
  _globals['_CONTAINSRESPONSE']._serialized_end=3506
  _globals['_INSERTREQUEST']._serialized_start=3508
  _globals['_INSERTREQUEST']._serialized_end=3573
  _globals['_INSERTRESPONSE']._serialized_start=3575
  _globals['_INSERTRESPONSE']._serialized_end=3679
  _globals['_UPDATEREQUEST']._serialized_start=3682
  _globals['_UPDATEREQUEST']._serialized_end=3825
  _globals['_UPDATERESPONSE']._serialized_start=3827
  _globals['_UPDATERESPONSE']._serialized_end=3931
  _globals['_REMOVEREQUEST']._serialized_start=3933
  _globals['_REMOVEREQUEST']._serialized_end=3998
  _globals['_REMOVERESPONSE']._serialized_start=4000
  _globals['_REMOVERESPONSE']._serialized_end=4104
  _globals['_TRUNCATECOLLECTIONREQUEST']._serialized_start=4106
  _globals['_TRUNCATECOLLECTIONREQUEST']._serialized_end=4158
  _globals['_TRUNCATECOLLECTIONRESPONSE']._serialized_start=4160
  _globals['_TRUNCATECOLLECTIONRESPONSE']._serialized_end=4276
  _globals['_LISTCOLLECTIONSREQUEST']._serialized_start=4278
  _globals['_LISTCOLLECTIONSREQUEST']._serialized_end=4302
  _globals['_LISTCOLLECTIONSRESPONSE']._serialized_start=4305
  _globals['_LISTCOLLECTIONSRESPONSE']._serialized_end=4444
  _globals['_DOCUMENTDBSERVICE']._serialized_start=4694
  _globals['_DOCUMENTDBSERVICE']._serialized_end=6292
# @@protoc_insertion_point(module_scope)



#
# Cognica
#
# Copyright (c) 2023 Cognica, Inc.
#

# pylint: disable=no-member,broad-exception-caught
# pylint: disable=missing-class-docstring,missing-function-docstring

import io
import time
import typing as t

import grpc
import numpy as np
from PIL import Image

from cognica.channel import Channel
from cognica.protobuf import (
    sentence_transformer_pb2,
    sentence_transformer_pb2_grpc,
)


messages: t.TypeAlias = sentence_transformer_pb2  # type: ignore

SentenceTransformerServiceStub = (
    sentence_transformer_pb2_grpc.SentenceTransformerServiceStub
)
CrossEncoderServiceStub = sentence_transformer_pb2_grpc.CrossEncoderServiceStub
CLIPEncoderServiceStub = sentence_transformer_pb2_grpc.CLIPEncoderServiceStub
QAEncoderServiceStub = sentence_transformer_pb2_grpc.QAEncoderServiceStub

StatusType: t.TypeAlias = messages.StatusType  # type: ignore
Tensor: t.TypeAlias = messages.Tensor  # type: ignore
SentenceEncoderRequest: t.TypeAlias = (
    messages.SentenceEncoderRequest  # type: ignore
)
SentenceEncoderResponse: t.TypeAlias = (
    messages.SentenceEncoderResponse  # type: ignore
)
SentencePair: t.TypeAlias = messages.SentencePair  # type: ignore
CrossEncoderRequest: t.TypeAlias = messages.CrossEncoderRequest  # type: ignore
CrossEncoderResponse: t.TypeAlias = (
    messages.CrossEncoderResponse  # type: ignore
)
CLIPEncoderRequest: t.TypeAlias = messages.CLIPEncoderRequest  # type: ignore
CLIPEncoderResponse: t.TypeAlias = messages.CLIPEncoderResponse  # type: ignore
QAEncoderRequest: t.TypeAlias = messages.QAEncoderRequest  # type: ignore
QAEncoderResponse: t.TypeAlias = messages.QAEncoderResponse  # type: ignore


def _create_stub(stub: t.Callable, channel: Channel):
    return stub(channel.channel)


class SentenceTransformerEncoder:
    def __init__(
        self, channel: Channel, model_name: str, timeout: int | None = None
    ):
        self._model_name = model_name
        self._max_retry_count = 5
        self._stub = _create_stub(SentenceTransformerServiceStub, channel)
        self._timeout = timeout

    def encode(self, sentences: t.List[str] | str) -> t.Optional[np.ndarray]:
        if not isinstance(sentences, list):
            sentences = [sentences]

        req = SentenceEncoderRequest(
            model_name=self._model_name, sentences=sentences
        )
        try:
            resp: SentenceEncoderResponse = self._invoke(
                self._stub.encode, req, wait_for_ready=True
            )
        except Exception:
            return None

        if resp.status != StatusType.kOK:
            return None

        data = []
        for tensor in resp.tensors:
            data.append(tensor.data)

        return np.array(data, dtype=np.float32)

    def _invoke(self, func, *args, **kwargs):
        retry = 0
        backoff = 0.1
        resp = None
        if self._timeout:
            kwargs["timeout"] = self._timeout
        while retry < self._max_retry_count:
            try:
                resp = func(*args, **kwargs)
            except grpc.RpcError as rpc_error:
                if rpc_error.code() == grpc.StatusCode.UNAVAILABLE:  # type: ignore
                    retry += 1
                    time.sleep(backoff)
                    backoff *= 2
                else:
                    raise
            else:
                break

        return resp


class SentenceTransformerCrossEncoder:
    def __init__(
        self, channel: Channel, model_name: str, timeout: int | None = None
    ):
        self._model_name = model_name
        self._max_retry_count = 5
        self._stub = _create_stub(CrossEncoderServiceStub, channel)
        self._timeout = timeout

    def predict(
        self, sentence_pairs: t.List[t.Tuple[str, str]]
    ) -> t.Optional[np.ndarray]:
        if not isinstance(sentence_pairs, list):
            sentence_pairs = [sentence_pairs]

        sentences = []
        for s1, s2 in sentence_pairs:
            pair = SentencePair(sentence1=s1, sentence2=s2)
            sentences.append(pair)
        req = CrossEncoderRequest(
            model_name=self._model_name, sentences=sentences
        )
        try:
            resp: CrossEncoderResponse = self._invoke(
                self._stub.predict, req, wait_for_ready=True
            )
        except Exception:
            return None

        if resp.status != StatusType.kOK:
            return None

        return np.array(resp.scores, dtype=np.float32)

    def _invoke(self, func, *args, **kwargs):
        retry = 0
        backoff = 0.1
        resp = None
        if self._timeout:
            kwargs["timeout"] = self._timeout
        while retry < self._max_retry_count:
            try:
                resp = func(*args, **kwargs)
            except grpc.RpcError as rpc_error:
                if rpc_error.code() == grpc.StatusCode.UNAVAILABLE:  # type: ignore
                    retry += 1
                    time.sleep(backoff)
                    backoff *= 2
                else:
                    raise
            else:
                break

        return resp


class SentenceTransformerCLIPEncoder:
    def __init__(
        self, channel: Channel, model_name: str, timeout: int | None = None
    ):
        self._model_name = model_name
        self._max_retry_count = 5
        self._stub = _create_stub(CLIPEncoderServiceStub, channel)
        self._timeout = timeout

    def encode(
        self, inputs: t.List[t.Union[bytes, Image.Image]]
    ) -> t.Optional[np.ndarray]:
        if not isinstance(inputs, list):
            inputs = [inputs]

        new_inputs = []
        for s in inputs:
            if isinstance(s, Image.Image):
                buffer = io.BytesIO()
                s.save(buffer, format="png")
                new_inputs.append(buffer.getvalue())
            else:
                new_inputs.append(s)

        req = CLIPEncoderRequest(model_name=self._model_name, inputs=new_inputs)
        try:
            resp: CLIPEncoderResponse = self._invoke(
                self._stub.encode, req, wait_for_ready=True
            )
        except Exception:
            return None

        if resp.status != StatusType.kOK:
            return None

        data = []
        for tensor in resp.tensors:
            data.append(tensor.data)

        return np.array(data, dtype=np.float32)

    def _invoke(self, func, *args, **kwargs):
        retry = 0
        backoff = 0.1
        resp = None
        if self._timeout:
            kwargs["timeout"] = self._timeout
        while retry < self._max_retry_count:
            try:
                resp = func(*args, **kwargs)
            except grpc.RpcError as rpc_error:
                if rpc_error.code() == grpc.StatusCode.UNAVAILABLE:  # type: ignore
                    retry += 1
                    time.sleep(backoff)
                    backoff *= 2
                else:
                    raise
            else:
                break

        return resp


class SentenceTransformerQAEncoder:
    def __init__(
        self, channel: Channel, model_name: str, timeout: int | None = None
    ):
        self._model_name = model_name
        self._max_retry_count = 5
        self._stub = _create_stub(QAEncoderServiceStub, channel)
        self._timeout = timeout

    def predict(
        self,
        questions: t.Union[str, t.List[str]],
        contexts: t.Union[str, t.List[str]],
        top_k: int = 1,
    ) -> t.Optional[t.List[t.List[t.Dict[str, t.Any]]]]:
        if not isinstance(questions, list):
            questions = [questions]
        if not isinstance(contexts, list):
            contexts = [contexts]

        req = QAEncoderRequest(
            model_name=self._model_name,
            questions=questions,
            contexts=contexts,
            top_k=top_k,
        )
        try:
            resp: QAEncoderResponse = self._invoke(
                self._stub.predict, req, wait_for_ready=True
            )
        except Exception:
            return None

        if resp.status != StatusType.kOK:
            return None

        outputs: t.List[t.List[t.Dict[str, t.Any]]] = []
        for answer in resp.answers:
            candidates: t.List[t.Dict[str, t.Any]] = []
            for candidate in answer.candidates:
                candidates.append(
                    {
                        "score": candidate.score,
                        "begin": candidate.begin,
                        "end": candidate.end,
                        "answer": candidate.answer,
                    }
                )
            outputs.append(candidates)

        return outputs

    def _invoke(self, func, *args, **kwargs):
        retry = 0
        backoff = 0.1
        resp = None
        if self._timeout:
            kwargs["timeout"] = self._timeout
        while retry < self._max_retry_count:
            try:
                resp = func(*args, **kwargs)
            except grpc.RpcError as rpc_error:
                if rpc_error.code() == grpc.StatusCode.UNAVAILABLE:  # type: ignore
                    retry += 1
                    time.sleep(backoff)
                    backoff *= 2
                else:
                    raise
            else:
                break

        return resp



#
# Cognica
#
# Copyright (c) 2023 Cognica, Inc.
#

# pylint: disable=invalid-name
# pylint: disable=missing-class-docstring,missing-function-docstring

import grpc


_CHANNEL_OPTIONS = [
    ("grpc.max_concurrent_streams", 4),
    ("grpc.max_send_message_length", -1),
    ("grpc.max_receive_message_length", -1),
    ("grpc.http2.max_frame_size", 10 * 1024 * 1024),  # 10MB
]


class Channel:
    def __init__(
        self,
        host: str,
        port: int,
        options: list[tuple[str, int]] | None = None,
        use_ssl: bool = False,
    ):
        self._host = host
        self._port = port
        if options is None:
            self._options = _CHANNEL_OPTIONS
        else:
            self._options = options
        self._use_ssl = use_ssl

        self._channel = self._create_channel()

    @property
    def host(self) -> str:
        return self._host

    @property
    def port(self) -> int:
        return self._port

    @property
    def options(self) -> list[tuple[str, int]]:
        return self._options

    @property
    def use_ssl(self) -> bool:
        return self._use_ssl

    @property
    def channel(self) -> grpc.Channel:
        return self._channel

    def _create_channel(self) -> grpc.Channel:
        if self._use_ssl:
            ChannelFactory = grpc.secure_channel
            params = {"credentials": grpc.ssl_channel_credentials()}
        else:
            ChannelFactory = grpc.insecure_channel
            params = {}

        channel = ChannelFactory(
            f"{self._host}:{self._port}",
            options=self._options,
            compression=grpc.Compression.Gzip,
            **params,
        )

        return channel

    def __enter__(self) -> grpc.Channel:
        return self._channel

    def __exit__(self, exc_type, exc_val, exc_tb):
        self._channel.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: key_value_db.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12key_value_db.proto\x12\x11\x63ognica.rpc.db.kv\"\"\n\x0bProfileInfo\x12\x13\n\x0b\x64uration_us\x18\x01 \x01(\x04\"{\n\x08Response\x12-\n\x06status\x18\x01 \x01(\x0e\x32\x1d.cognica.rpc.db.kv.StatusType\x12\x0f\n\x07message\x18\x02 \x01(\t\x12/\n\x07profile\x18\x03 \x01(\x0b\x32\x1e.cognica.rpc.db.kv.ProfileInfo\"g\n\nPutRequest\x12\x15\n\rkeyspace_name\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\x0c\x12\r\n\x05value\x18\x03 \x01(\x0c\x12\x0b\n\x03ttl\x18\x04 \x01(\x03\x12\x19\n\x11\x63reate_if_missing\x18\x05 \x01(\x08\"<\n\x0bPutResponse\x12-\n\x08response\x18\x01 \x01(\x0b\x32\x1b.cognica.rpc.db.kv.Response\"3\n\rRemoveRequest\x12\x15\n\rkeyspace_name\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\x0c\"?\n\x0eRemoveResponse\x12-\n\x08response\x18\x01 \x01(\x0b\x32\x1b.cognica.rpc.db.kv.Response\"0\n\nGetRequest\x12\x15\n\rkeyspace_name\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\x0c\"K\n\x0bGetResponse\x12-\n\x08response\x18\x01 \x01(\x0b\x32\x1b.cognica.rpc.db.kv.Response\x12\r\n\x05value\x18\x02 \x01(\x0c\"6\n\x0fMultiGetRequest\x12\x15\n\rkeyspace_name\x18\x01 \x01(\t\x12\x0c\n\x04keys\x18\x02 \x03(\x0c\"R\n\x10MultiGetResponse\x12.\n\tresponses\x18\x01 \x03(\x0b\x32\x1b.cognica.rpc.db.kv.Response\x12\x0e\n\x06values\x18\x02 \x03(\x0c\"q\n\x11\x42\x61tchedPutRequest\x12\x15\n\rkeyspace_name\x18\x01 \x01(\t\x12\x0c\n\x04keys\x18\x02 \x03(\x0c\x12\x0e\n\x06values\x18\x03 \x03(\x0c\x12\x0c\n\x04ttls\x18\x04 \x03(\x03\x12\x19\n\x11\x63reate_if_missing\x18\x05 \x01(\x08\"D\n\x12\x42\x61tchedPutResponse\x12.\n\tresponses\x18\x01 \x03(\x0b\x32\x1b.cognica.rpc.db.kv.Response\";\n\x14\x42\x61tchedRemoveRequest\x12\x15\n\rkeyspace_name\x18\x01 \x01(\t\x12\x0c\n\x04keys\x18\x02 \x03(\x0c\"G\n\x15\x42\x61tchedRemoveResponse\x12.\n\tresponses\x18\x01 \x03(\x0b\x32\x1b.cognica.rpc.db.kv.Response\"8\n\x11\x42\x61tchedGetRequest\x12\x15\n\rkeyspace_name\x18\x01 \x01(\t\x12\x0c\n\x04keys\x18\x02 \x03(\x0c\"T\n\x12\x42\x61tchedGetResponse\x12.\n\tresponses\x18\x01 \x03(\x0b\x32\x1b.cognica.rpc.db.kv.Response\x12\x0e\n\x06values\x18\x02 \x03(\x0c\".\n\x15\x43reateKeyspaceRequest\x12\x15\n\rkeyspace_name\x18\x01 \x01(\t\"G\n\x16\x43reateKeyspaceResponse\x12-\n\x08response\x18\x01 \x01(\x0b\x32\x1b.cognica.rpc.db.kv.Response\",\n\x13\x44ropKeyspaceRequest\x12\x15\n\rkeyspace_name\x18\x01 \x01(\t\"E\n\x14\x44ropKeyspaceResponse\x12-\n\x08response\x18\x01 \x01(\x0b\x32\x1b.cognica.rpc.db.kv.Response\"0\n\x17TruncateKeyspaceRequest\x12\x15\n\rkeyspace_name\x18\x01 \x01(\t\"I\n\x18TruncateKeyspaceResponse\x12-\n\x08response\x18\x01 \x01(\x0b\x32\x1b.cognica.rpc.db.kv.Response\"\x16\n\x14ListKeyspacesRequest\"^\n\x15ListKeyspacesResponse\x12-\n\x08response\x18\x01 \x01(\x0b\x32\x1b.cognica.rpc.db.kv.Response\x12\x16\n\x0ekeyspace_names\x18\x02 \x03(\t*3\n\nStatusType\x12\x07\n\x03kOK\x10\x00\x12\r\n\tkNotFound\x10\x01\x12\r\n\tkInternal\x10\n2\xe4\x04\n\x11KeyValueDBService\x12\x46\n\x03put\x12\x1d.cognica.rpc.db.kv.PutRequest\x1a\x1e.cognica.rpc.db.kv.PutResponse\"\x00\x12O\n\x06remove\x12 .cognica.rpc.db.kv.RemoveRequest\x1a!.cognica.rpc.db.kv.RemoveResponse\"\x00\x12\x46\n\x03get\x12\x1d.cognica.rpc.db.kv.GetRequest\x1a\x1e.cognica.rpc.db.kv.GetResponse\"\x00\x12Q\n\x04mget\x12\".cognica.rpc.db.kv.MultiGetRequest\x1a#.cognica.rpc.db.kv.MultiGetResponse\"\x00\x12Z\n\tput_batch\x12$.cognica.rpc.db.kv.BatchedPutRequest\x1a%.cognica.rpc.db.kv.BatchedPutResponse\"\x00\x12\x63\n\x0cremove_batch\x12\'.cognica.rpc.db.kv.BatchedRemoveRequest\x1a(.cognica.rpc.db.kv.BatchedRemoveResponse\"\x00\x12Z\n\tget_batch\x12$.cognica.rpc.db.kv.BatchedGetRequest\x1a%.cognica.rpc.db.kv.BatchedGetResponse\"\x00\x32\xbd\x03\n\x16KeyspaceManagerService\x12h\n\x0f\x63reate_keyspace\x12(.cognica.rpc.db.kv.CreateKeyspaceRequest\x1a).cognica.rpc.db.kv.CreateKeyspaceResponse\"\x00\x12\x62\n\rdrop_keyspace\x12&.cognica.rpc.db.kv.DropKeyspaceRequest\x1a\'.cognica.rpc.db.kv.DropKeyspaceResponse\"\x00\x12n\n\x11truncate_keyspace\x12*.cognica.rpc.db.kv.TruncateKeyspaceRequest\x1a+.cognica.rpc.db.kv.TruncateKeyspaceResponse\"\x00\x12\x65\n\x0elist_keyspaces\x12\'.cognica.rpc.db.kv.ListKeyspacesRequest\x1a(.cognica.rpc.db.kv.ListKeyspacesResponse\"\x00\x42\x03\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'key_value_db_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\370\001\001'
  _STATUSTYPE._serialized_start=1700
  _STATUSTYPE._serialized_end=1751
  _PROFILEINFO._serialized_start=41
  _PROFILEINFO._serialized_end=75
  _RESPONSE._serialized_start=77
  _RESPONSE._serialized_end=200
  _PUTREQUEST._serialized_start=202
  _PUTREQUEST._serialized_end=305
  _PUTRESPONSE._serialized_start=307
  _PUTRESPONSE._serialized_end=367
  _REMOVEREQUEST._serialized_start=369
  _REMOVEREQUEST._serialized_end=420
  _REMOVERESPONSE._serialized_start=422
  _REMOVERESPONSE._serialized_end=485
  _GETREQUEST._serialized_start=487
  _GETREQUEST._serialized_end=535
  _GETRESPONSE._serialized_start=537
  _GETRESPONSE._serialized_end=612
  _MULTIGETREQUEST._serialized_start=614
  _MULTIGETREQUEST._serialized_end=668
  _MULTIGETRESPONSE._serialized_start=670
  _MULTIGETRESPONSE._serialized_end=752
  _BATCHEDPUTREQUEST._serialized_start=754
  _BATCHEDPUTREQUEST._serialized_end=867
  _BATCHEDPUTRESPONSE._serialized_start=869
  _BATCHEDPUTRESPONSE._serialized_end=937
  _BATCHEDREMOVEREQUEST._serialized_start=939
  _BATCHEDREMOVEREQUEST._serialized_end=998
  _BATCHEDREMOVERESPONSE._serialized_start=1000
  _BATCHEDREMOVERESPONSE._serialized_end=1071
  _BATCHEDGETREQUEST._serialized_start=1073
  _BATCHEDGETREQUEST._serialized_end=1129
  _BATCHEDGETRESPONSE._serialized_start=1131
  _BATCHEDGETRESPONSE._serialized_end=1215
  _CREATEKEYSPACEREQUEST._serialized_start=1217
  _CREATEKEYSPACEREQUEST._serialized_end=1263
  _CREATEKEYSPACERESPONSE._serialized_start=1265
  _CREATEKEYSPACERESPONSE._serialized_end=1336
  _DROPKEYSPACEREQUEST._serialized_start=1338
  _DROPKEYSPACEREQUEST._serialized_end=1382
  _DROPKEYSPACERESPONSE._serialized_start=1384
  _DROPKEYSPACERESPONSE._serialized_end=1453
  _TRUNCATEKEYSPACEREQUEST._serialized_start=1455
  _TRUNCATEKEYSPACEREQUEST._serialized_end=1503
  _TRUNCATEKEYSPACERESPONSE._serialized_start=1505
  _TRUNCATEKEYSPACERESPONSE._serialized_end=1578
  _LISTKEYSPACESREQUEST._serialized_start=1580
  _LISTKEYSPACESREQUEST._serialized_end=1602
  _LISTKEYSPACESRESPONSE._serialized_start=1604
  _LISTKEYSPACESRESPONSE._serialized_end=1698
  _KEYVALUEDBSERVICE._serialized_start=1754
  _KEYVALUEDBSERVICE._serialized_end=2366
  _KEYSPACEMANAGERSERVICE._serialized_start=2369
  _KEYSPACEMANAGERSERVICE._serialized_end=2814
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: fts_analysis_pipeline.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1b\x66ts_analysis_pipeline.proto\x12\x12\x63ognica.rpc.db.fts\"\"\n\x0bProfileInfo\x12\x13\n\x0b\x64uration_us\x18\x01 \x01(\x04\"k\n\x18PipelineExecutionRequest\x12\x17\n\x0f\x63ollection_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\x12\x13\n\x0b\x66ield_names\x18\x03 \x03(\t\x12\r\n\x05query\x18\x04 \x01(\t\"m\n\x19PipelineExecutionResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0e\n\x06result\x18\x02 \x01(\t\x12\x30\n\x07profile\x18\x03 \x01(\x0b\x32\x1f.cognica.rpc.db.fts.ProfileInfo\"Y\n\x1d\x41\x64hocPipelineExecutionRequest\x12\x14\n\x0cpipeline_def\x18\x01 \x01(\t\x12\x13\n\x0b\x66ield_names\x18\x02 \x03(\t\x12\r\n\x05query\x18\x03 \x01(\t\"r\n\x1e\x41\x64hocPipelineExecutionResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0e\n\x06result\x18\x02 \x01(\t\x12\x30\n\x07profile\x18\x03 \x01(\x0b\x32\x1f.cognica.rpc.db.fts.ProfileInfo2\x80\x02\n\x1a\x46TSAnalysisPipelineService\x12h\n\x07\x65xecute\x12,.cognica.rpc.db.fts.PipelineExecutionRequest\x1a-.cognica.rpc.db.fts.PipelineExecutionResponse\"\x00\x12x\n\rexecute_adhoc\x12\x31.cognica.rpc.db.fts.AdhocPipelineExecutionRequest\x1a\x32.cognica.rpc.db.fts.AdhocPipelineExecutionResponse\"\x00\x42\x03\xf8\x01\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'fts_analysis_pipeline_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\370\001\001'
  _PROFILEINFO._serialized_start=51
  _PROFILEINFO._serialized_end=85
  _PIPELINEEXECUTIONREQUEST._serialized_start=87
  _PIPELINEEXECUTIONREQUEST._serialized_end=194
  _PIPELINEEXECUTIONRESPONSE._serialized_start=196
  _PIPELINEEXECUTIONRESPONSE._serialized_end=305
  _ADHOCPIPELINEEXECUTIONREQUEST._serialized_start=307
  _ADHOCPIPELINEEXECUTIONREQUEST._serialized_end=396
  _ADHOCPIPELINEEXECUTIONRESPONSE._serialized_start=398
  _ADHOCPIPELINEEXECUTIONRESPONSE._serialized_end=512
  _FTSANALYSISPIPELINESERVICE._serialized_start=515
  _FTSANALYSISPIPELINESERVICE._serialized_end=771
# @@protoc_insertion_point(module_scope)



#
# Cognica
#
# Copyright (c) 2023 Cognica, Inc.
#

from cognica.channel import Channel
from cognica.document_db import DocumentDB
from cognica.fts_analysis_pipeline import FTSAnalysisPipeline
from cognica.key_value_db import KeyValueDB, KeyspaceManager
from cognica.sentence_transformer import (
    SentenceTransformerEncoder,
    SentenceTransformerCrossEncoder,
    SentenceTransformerCLIPEncoder,
    SentenceTransformerQAEncoder,
)


__all__ = [
    "Channel",
    "DocumentDB",
    "FTSAnalysisPipeline",
    "KeyValueDB",
    "KeyspaceManager",
    "SentenceTransformerEncoder",
    "SentenceTransformerCrossEncoder",
    "SentenceTransformerCLIPEncoder",
    "SentenceTransformerQAEncoder",
]



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import cognica.protobuf.fts_analysis_pipeline_pb2 as fts__analysis__pipeline__pb2


class FTSAnalysisPipelineServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.execute = channel.unary_unary(
                '/cognica.rpc.db.fts.FTSAnalysisPipelineService/execute',
                request_serializer=fts__analysis__pipeline__pb2.PipelineExecutionRequest.SerializeToString,
                response_deserializer=fts__analysis__pipeline__pb2.PipelineExecutionResponse.FromString,
                )
        self.execute_adhoc = channel.unary_unary(
                '/cognica.rpc.db.fts.FTSAnalysisPipelineService/execute_adhoc',
                request_serializer=fts__analysis__pipeline__pb2.AdhocPipelineExecutionRequest.SerializeToString,
                response_deserializer=fts__analysis__pipeline__pb2.AdhocPipelineExecutionResponse.FromString,
                )


class FTSAnalysisPipelineServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def execute(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def execute_adhoc(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_FTSAnalysisPipelineServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'execute': grpc.unary_unary_rpc_method_handler(
                    servicer.execute,
                    request_deserializer=fts__analysis__pipeline__pb2.PipelineExecutionRequest.FromString,
                    response_serializer=fts__analysis__pipeline__pb2.PipelineExecutionResponse.SerializeToString,
            ),
            'execute_adhoc': grpc.unary_unary_rpc_method_handler(
                    servicer.execute_adhoc,
                    request_deserializer=fts__analysis__pipeline__pb2.AdhocPipelineExecutionRequest.FromString,
                    response_serializer=fts__analysis__pipeline__pb2.AdhocPipelineExecutionResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'cognica.rpc.db.fts.FTSAnalysisPipelineService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class FTSAnalysisPipelineService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def execute(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.fts.FTSAnalysisPipelineService/execute',
            fts__analysis__pipeline__pb2.PipelineExecutionRequest.SerializeToString,
            fts__analysis__pipeline__pb2.PipelineExecutionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def execute_adhoc(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/cognica.rpc.db.fts.FTSAnalysisPipelineService/execute_adhoc',
            fts__analysis__pipeline__pb2.AdhocPipelineExecutionRequest.SerializeToString,
            fts__analysis__pipeline__pb2.AdhocPipelineExecutionResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)
